#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG;

use strict;
use SiteDB;

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self = shift || {};
    
    if (! defined $self->{SITEDB}) {
        $self->{SITEDB} = SiteDB->new();
    } else {
        $self->{SITENAME} = $self->{SITEDB}->siteName if (! defined $self->{SITENAME});
    }
    
    bless ($self, $class);
    $self;
}

sub error($) {
    my $self = shift;
    my $msg = shift;
    print "ERROR: $msg\n";
    1;
}

sub warning($) {
    my $self = shift;
    my $msg = shift;
    print "WARNING: $msg\n" if (exists $self->{WARNING});
    1;
}

sub debug($) {
    my $self = shift;
    my $msg = shift;
    print "DEBUG: $msg\n" if (exists $self->{DEBUG});
    1;
}

sub verbose($) {
    my $self = shift;
    my $msg = shift;
    print "$msg\n" if (exists $self->{VERBOSE});
    1;
}

sub checkPing ($) {
    use Net::Ping;
    
	my $self = shift;
    my $host = shift;
    my $retVal;

	my $ping = new Net::Ping("icmp", 10);

    if ($ping->ping($host)) {
        $retVal = 1;
    } else {
        $retVal = 0;
    }
	
	$ping->close();

    $retVal;
}

sub checkTcp {
    use Net::Ping;
    
    my $self = shift;
    my $host = shift;
	my $port = shift;
    my $retVal;

	my $ping = new Net::Ping("tcp", 10);
	$ping->{port_num} = $port;
    $ping->service_check(1);
    
    if ($ping->ping($host)) {
        $retVal = 1;
    } else {
        $retVal = 0;
    }
	
	$ping->close();

    $retVal;
}

sub getData
{
    my $self = shift;
    $self->error("Method getData not implemented!");
}


=head1 NAME

NCG

=head1 DESCRIPTION

The NCG module is top level class for all modules responsible for
extracting information about grid site, services, metricsets &
local/remote probes used for monitoring.

Main method for invoking module's functionality is getData. Each 
module must have this method implemented.

NCG contains helper functions useful to extending modules: logging,
port and host checking, etc.

=head1 SYNOPSIS

  use NCG;

=cut

=head1 METHODS

=over

=item C<new>

  $dbh = NCG->new( $attr );

Creates new NCG instance.

=item C<checkPing>

  $res = $ncg->checkPing($host);

Method checks if host is reachable with ICMP.

=item C<checkTcp>

  $res = $ncg->checkTcp($host, $port);

Method checks if TCP port on specified host is opened.

=item C<error>

  $res = $ncg->error($msg);

Method prints error message. This method is used for logging purposes.

=item C<warning>

  $res = $ncg->warning($msg);

Method prints warning message. This method is used for logging purposes.

=item C<verbose>

  $res = $ncg->verbose($msg);

Method prints message if verbose is switched on. This method is used
for logging purposes.

=back

=head1 SEE ALSO

=cut


1;
