#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::ConfigGen::Nagios;

use strict;
use NCG::ConfigGen;
use Socket;
use Sys::Hostname;
use Date::Format;

use vars qw(@ISA);

@ISA=("NCG::ConfigGen");

my $DEFAULT_TEMPLATES_DIR = '/opt/lcg/share/grid-monitoring/config-gen/nagios';
my $DEFAULT_OUTPUT_DIR = '/etc/nagios/wlcg.d';
my $DEFAULT_NRPE_OUTPUT_DIR = '/etc/nagios';

my $DEFAULT_NRPE_WLCG_PLUGINS_DIR = '/usr/lib/nagios/plugins/wlcg';
my $DEFAULT_NRPE_WLCG_PROBES_DIR = '/opt/lcg/share/grid-monitoring/probes';
my $DEFAULT_NRPE_PROXY_FILE = '/etc/nagios/globus/userproxy.pem';
my $DEFAULT_NRPE_PASSPHRASE_FILE = '/etc/nagios/globus/proxy_passphrase';
my $DEFAULT_NRPE_MYPROXY_NAME = 'NagiosRefresh';

my $DEFAULT_VO = 'dteam';

my $CONFIGURATION_TEMPLATES =
   {
       CONTACTS                 => 'contacts.template',
       HOSTS                    => 'hosts.template',
       SERVICES                 => 'services.template',
       COMMANDS                 => 'commands.template',
       NAGIOS                   => 'wlcg.nagios.template',
       CONTACTS_CONTACT         => 'contacts/contact.template',
       CONTACTS_CONTACTGROUP    => 'contacts/contactgroup.template',
       HOSTS_HOST               => 'hosts/host.template',
       HOSTS_HOSTGROUP          => 'hosts/hostgroup.template',
       SERVICES_SERVICEGROUP    => 'services/servicegroup.template',
       SERVICES_SERVICEDEP      => 'services/servicedependency.template',
       SERVICES_NATIVE          => 'services/native.template',
       SERVICES_WLCG            => 'services/wlcg.template',
       SERVICES_WLCGNRPE        => 'services/wlcg.nrpe.template',
       SERVICES_NPM             => 'services/npm.template',
       SERVICES_SAM             => 'services/sam.template',
       NAGIOS_CADIST            => 'wlcg.nagios/cadist.template',
       NAGIOS_GRIDPROXY         => 'wlcg.nagios/gridproxy.template',
       NAGIOS_MYPROXY           => 'wlcg.nagios/myproxy.template',
       NAGIOS_NPM               => 'wlcg.nagios/npm.template',
       NAGIOS_SAM               => 'wlcg.nagios/sam.template',
       NRPE_CADIST              => 'wlcg.nagios/cadist.nrpe.template',
       NRPE_GRIDPROXY           => 'wlcg.nagios/gridproxy.nrpe.template',
       NRPE_MYPROXY             => 'wlcg.nagios/myproxy.nrpe.template',
       NRPE_NRPE                => 'wlcg.nagios/nrpe.template',
       NRPE_LOCALCOMMANDS       => 'wlcg.nrpe/local.commands.template',
       NRPE_WLCGCOMMANDS        => 'wlcg.nrpe/wlcg.commands.template',
   };

my $CONFIGURATION_OUTPUT =
   {
       CONTACTS                 => 'contacts.cfg',
       HOSTS                    => 'hosts.cfg',
       SERVICES                 => 'services.cfg',
       COMMANDS                 => 'commands.cfg',
       NAGIOS                   => 'wlcg.nagios.cfg',
   };
   
my $NRPE_CONFIGURATION_OUTPUT =
   {
       NRPE                     => 'wlcg.nrpe.cfg'
   };

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);

    # set default values
    if (! defined $self->{TEMPLATES_DIR}) {
        $self->{TEMPLATES_DIR} = $DEFAULT_TEMPLATES_DIR;
    }

    if (! defined $self->{OUTPUT_DIR}) {
        $self->{OUTPUT_DIR} = $DEFAULT_OUTPUT_DIR;
    }

    if (! defined $self->{NRPE_OUTPUT_DIR}) {
        $self->{NRPE_OUTPUT_DIR} = $DEFAULT_NRPE_OUTPUT_DIR;
    }

    if (! defined $self->{USER_OFFSET}) {
        $self->{USER_OFFSET} = 2;
    }

    if (! defined $self->{GLITE_VERSION}) {
        $self->{GLITE_VERSION} = "UNKNOWN";
    }

    if (! defined $self->{SAM_INCLUDE_ALL}) {
        $self->{SAM_INCLUDE_ALL} = 0;
    }

    if (! defined $self->{NAGIOS_ADMIN}) {
        $self->{NAGIOS_ADMIN} = 'root@localhost';
    }
    
    if (! defined $self->{INCLUDE_HOSTS}) {
        $self->{INCLUDE_HOSTS} = 1;
    }

    if (! defined $self->{NRPE_WLCG_PLUGINS_DIR}) {
        $self->{NRPE_WLCG_PLUGINS_DIR} = $DEFAULT_NRPE_WLCG_PLUGINS_DIR;
    }

    if (! defined $self->{NRPE_WLCG_PROBES_DIR}) {
        $self->{NRPE_WLCG_PROBES_DIR} = $DEFAULT_NRPE_WLCG_PROBES_DIR;
    }

    if (! defined $self->{NRPE_PROXY_FILE}) {
        $self->{NRPE_PROXY_FILE} = $DEFAULT_NRPE_PROXY_FILE;
    }

    if (! defined $self->{NRPE_PASSPHRASE_FILE}) {
        $self->{NRPE_PASSPHRASE_FILE} = $DEFAULT_NRPE_PASSPHRASE_FILE;
    }

    if (! defined $self->{NRPE_MYPROXY_NAME}) {
        $self->{NRPE_MYPROXY_NAME} = $DEFAULT_NRPE_MYPROXY_NAME;
    }

    if (! defined $self->{VO}) {
        $self->{VO} = $DEFAULT_VO;
    }

    if (! $self->_checkProbesType) {
        undef $self;
        return;
    }

    if (! $self->_checkNrpeUI ) {
        undef $self;
        return;
    }
    
    if (! $self->_checkMyProxyServer ) {
        undef $self;
        return;
    }
    
    if (! $self->_checkNagiosServer ) {
        undef $self;
        return;
    }

    if (! $self->_checkFiles ) {
        undef $self;
        return;
    }
    
    $self;
}

#############################
##
##  Constructor helpers
##
#############################


sub _checkProbesType {
    my $self = shift;

    if (! defined $self->{PROBES_TYPE}) {
        $self->{PROBES_TYPE} = 'all';
    } else {
        foreach my $pt (split (/,/, $self->{PROBES_TYPE})) {
            if ($pt eq 'all') {
                $self->{PROBES_TYPE} = 'all';
                last;
            } elsif (   $pt eq 'remote' ||
                        $pt eq 'local' ||
                        $pt eq 'native')
            {
                $self->{PROBES_TYPE_FLAG}->{$pt} = 1;
            } else {
                $self->error("Incorrect PROBES_TYPE value. Parameter PROBES_TYPE can be: all, remote, local or native.");
                return;
            }
        }
    }

    if ($self->{PROBES_TYPE} eq 'all') {
        $self->{PROBES_TYPE_FLAG}->{local} = 1;
        $self->{PROBES_TYPE_FLAG}->{remote} = 1;
        $self->{PROBES_TYPE_FLAG}->{native} = 1;
    }

    return 1;
}

sub _checkNrpeUI {
    my $self = shift;
    my $addr;

    if (! defined $self->{NRPE_UI} ) {
        $self->{NRPE_UI} = 0;
        return 1;
    }

    if (! defined $self->{PROBES_TYPE_FLAG}->{local} ) {
        $self->{NRPE_UI} = 0;
        return 1;
    }

    if (! $self->{NRPE_UI}) {
        return 1;
    }

    unless ($addr = scalar gethostbyname($self->{NRPE_UI})) {
        $self->error("Invalid hostname for NRPE_UI: $self->{NRPE_UI}.");
        return;
    }

    $self->{NRPE_UI_ADDRESS} = inet_ntoa($addr);

    return 1;
}

sub _checkMyProxyServer {
    my $self = shift;
    my $addr;

    if (! defined $self->{MYPROXY_SERVER} ) {
        if ($self->{PROBES_TYPE_FLAG}->{local}) {
            $self->error("Parameter MYPROXY_SERVER must be defined when local probes are used.");
            return;
        }
        return 1;
    }

    unless ($addr = scalar gethostbyname($self->{MYPROXY_SERVER})) {
        $self->error("Invalid hostname for MYPROXY_SERVER: $self->{MYPROXY_SERVER}.");
        return;
    }

    $self->{MYPROXY_SERVER_ADDRESS} = inet_ntoa($addr);

    return 1;
}

sub _checkNagiosServer {
    my $self = shift;
    my $addr;

    if (! defined $self->{NAGIOS_SERVER}) {
        $self->{NAGIOS_SERVER} = hostname();
        $self->{NAGIOS_SERVER_ADDRESS} = inet_ntoa(scalar gethostbyname($self->{NAGIOS_SERVER}));
        return 1;
    }

    unless ($addr = scalar gethostbyname($self->{NAGIOS_SERVER})) {
        $self->error("Invalid hostname for NAGIOS_SERVER: $self->{NAGIOS_SERVER}.");
        return;
    }

    $self->{NAGIOS_SERVER_ADDRESS} = inet_ntoa($addr);
    
    return 1;
}


sub _checkFiles {
    my $self = shift;

    if ( ! -d $self->{TEMPLATES_DIR} ) {
        $self->error("Templates directory ". $self->{TEMPLATES_DIR} ." does not exist.");
        return;
    }

    foreach my $config (keys %$CONFIGURATION_TEMPLATES) {
        $CONFIGURATION_TEMPLATES->{$config} = $self->{TEMPLATES_DIR} . "/" . $CONFIGURATION_TEMPLATES->{$config};
        if (! -f $CONFIGURATION_TEMPLATES->{$config}) {
            $self->error("Template ". $CONFIGURATION_TEMPLATES->{$config} ." does not exist.");
            return;
        }

        if (! -r $CONFIGURATION_TEMPLATES->{$config}) {
            $self->error("Template ". $CONFIGURATION_TEMPLATES->{$config} ." is not readable.");
            return;
        }
    }

    if ( ! -d $self->{OUTPUT_DIR} ) {
        $self->warning("Output directory ". $self->{OUTPUT_DIR} ." does not exist. Creating output directory ...");
        my $res = `/bin/mkdir -p $self->{OUTPUT_DIR} 2>&1`;
        if ($?) {
            $self->error("Error creating directory ". $self->{OUTPUT_DIR} .": $res.");
            return;
        }
    }

    foreach my $output (keys %$CONFIGURATION_OUTPUT) {
        $CONFIGURATION_OUTPUT->{$output} = $self->{OUTPUT_DIR} . "/" . $CONFIGURATION_OUTPUT->{$output};
    }
    
    if ($self->{NRPE_UI}) {
        if ( ! -d $self->{NRPE_OUTPUT_DIR} ) {
            $self->warning("Output directory ". $self->{NRPE_OUTPUT_DIR} ." does not exist. Creating output directory ...");
            my $res = `/bin/mkdir -p $self->{NRPE_OUTPUT_DIR} 2>&1`;
            if ($?) {
                $self->error("Error creating directory ". $self->{NRPE_OUTPUT_DIR} .": $res.");
                return;
            }
        }
        foreach my $noutput (keys %$NRPE_CONFIGURATION_OUTPUT) {
            $NRPE_CONFIGURATION_OUTPUT->{$noutput} = $self->{NRPE_OUTPUT_DIR} . "/" . $NRPE_CONFIGURATION_OUTPUT->{$noutput};
        }
    }

    1;
}

#############################
##
##  Commands (commands.cfg)
##
#############################

sub _genCommands {
	my $self = shift;
    my $CONFIG;
    my $TEMPL;
    my $line;

    if (!open ($CONFIG, ">" . $CONFIGURATION_OUTPUT->{COMMANDS})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_OUTPUT->{COMMANDS}."!");
        return;
    }

    print $CONFIG $self->{OUTPUT_HEADER};

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{COMMANDS})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{COMMANDS}."!");
        close ($CONFIG);
        return;
    }

    my $user1 = "USER" . $self->{USER_OFFSET};
    my $user2 = "USER" . ($self->{USER_OFFSET}+1);

    while ($line = <$TEMPL>){
        $line =~ s/<USER_1>/$user1/g;
        $line =~ s/<USER_2>/$user2/g;
        $line =~ s/<NRPE_UI>/$self->{NRPE_UI}/g;
        print $CONFIG $line;
    }
    
    close ($TEMPL);
    close ($CONFIG);
    
    1;
}

#############################
##
##  Nagios internal checks (wlcg.nagios.cfg)
##
#############################

#############################
##  Nagios "remote" internal checks
#############################

sub _genNagiosRemoteInstances {
	my $self = shift;
    my $CONFIG = shift;
    my $TEMPL;
    my $line;
    my $samall = "";
    my $samoptions = "";
    my $sitename = $self->{SITEDB}->siteName();

    $samoptions .= " --sam-all" if ($self->{SAM_INCLUDE_ALL});
    $samoptions .= " --sam-root-url " . $self->{SAM_ROOT_URL} if ($self->{SAM_ROOT_URL});
    $samoptions .= " --vos " . $self->{SAM_VOS} if ($self->{SAM_VOS});

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NAGIOS_SAM})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NAGIOS_SAM}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<sitename>/$sitename/mg;
    $line =~ s/<samoptions>/$samoptions/mg;
    $line =~ s/<NAGIOS_SERVER>/$self->{NAGIOS_SERVER}/mg;
    print $CONFIG $line;


    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NAGIOS_NPM})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NAGIOS_NPM}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<sitename>/$sitename/mg;
    $line =~ s/<NAGIOS_SERVER>/$self->{NAGIOS_SERVER}/mg;
    print $CONFIG $line;

    1;
}

#############################
##  Nagios "local" internal checks
#############################

sub _genGridProxy {
	my $self = shift;
    my $CONFIG = shift;
    my $vo = shift;
    my $offset = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NAGIOS_GRIDPROXY})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NAGIOS_GRIDPROXY}."!");
        return;
    }

    my $user1 = "USER" . $offset;
    my $user2 = "USER" . ($offset+1);
    my $user3 = "USER" . ($offset+2);

    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<VONAME>/$vo/mg;
    $line =~ s/<NAGIOS_SERVER>/$self->{NAGIOS_SERVER}/mg;
    $line =~ s/<MYPROXY_SERVER>/$self->{MYPROXY_SERVER}/mg;
    $line =~ s/<USER_MYNAME>/$user1/mg;
    $line =~ s/<USER_PASS>/$user2/mg;
    $line =~ s/<USER_PROXY>/$user3/mg;
    print $CONFIG $line;

    1;
}

sub _genMyProxy {
	my $self = shift;
    my $CONFIG = shift;
    my $offset = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NAGIOS_MYPROXY})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NAGIOS_MYPROXY}."!");
        return;
    }

    my $user1 = "USER" . $offset;
    my $user3 = "USER" . ($offset+2);

    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<NAGIOS_SERVER>/$self->{NAGIOS_SERVER}/mg;
    $line =~ s/<MYPROXY_SERVER>/$self->{MYPROXY_SERVER}/mg;
    $line =~ s/<USER_MYNAME>/$user1/mg;
    $line =~ s/<USER_PROXY>/$user3/mg;
    print $CONFIG $line;

    1;
}

sub _genCADist {
	my $self = shift;
    my $CONFIG = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NAGIOS_CADIST})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NAGIOS_CADIST}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<NAGIOS_SERVER>/$self->{NAGIOS_SERVER}/mg;
    print $CONFIG $line;

    1;
}

#############################
##  Nagios "local" internal checks via NRPE
#############################

sub _genGridProxyNrpe {
	my $self = shift;
    my $CONFIG = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NRPE_GRIDPROXY})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NRPE_GRIDPROXY}."!");
        return;
    }

    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<NRPE_UI>/$self->{NRPE_UI}/mg;
    print $CONFIG $line;

    1;
}

sub _genMyProxyNrpe {
	my $self = shift;
    my $CONFIG = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NRPE_MYPROXY})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NRPE_MYPROXY}."!");
        return;
    }

    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<NRPE_UI>/$self->{NRPE_UI}/mg;
    $line =~ s/<MYPROXY_SERVER>/$self->{MYPROXY_SERVER}/mg;
    print $CONFIG $line;

    1;
}

sub _genCADistNrpe {
	my $self = shift;
    my $CONFIG = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NRPE_CADIST})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NRPE_CADIST}."!");
        return;
    }

    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<NRPE_UI>/$self->{NRPE_UI}/mg;
    print $CONFIG $line;

    1;
}

sub _genNrpe {
	my $self = shift;
    my $CONFIG = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NRPE_NRPE})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NRPE_NRPE}."!");
        return;
    }

    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<NRPE_UI>/$self->{NRPE_UI}/mg;
    print $CONFIG $line;

    1;
}

sub _genNagiosLocal {
	my $self = shift;
    my $CONFIG = shift;
	my $vo = shift;
    my $line;

    if (!$self->{NRPE_UI}) {
        if (!$self->_genCADist($CONFIG)) {
            return;
        }
        my $offset = $self->{USER_OFFSET} + 2;
        if (!$self->_genGridProxy($CONFIG, $vo, $offset)) {
            return;
        }
        if (!$self->_genMyProxy($CONFIG, $offset)) {
            return;
        }
    } else {
        if (!$self->_genCADistNrpe($CONFIG)) {
            return;
        }
        if (!$self->_genGridProxyNrpe($CONFIG)) {
            return;
        }
        if (!$self->_genMyProxyNrpe($CONFIG)) {
            return;
        }
        if (!$self->_genNrpe($CONFIG)) {
            return;
        }
    }

    1;
}

sub _genNagios {
	my $self = shift;
	my $vo = shift;
    my $CONFIG;
    my $TEMPL;
    my $line;
    
    if (!open ($CONFIG, ">" . $CONFIGURATION_OUTPUT->{NAGIOS})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_OUTPUT->{NAGIOS}."!");
        return;
    }

    print $CONFIG $self->{OUTPUT_HEADER};

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NAGIOS})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NAGIOS}."!");
        close ($CONFIG);
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    print $CONFIG $line;

    if ($self->{PROBES_TYPE_FLAG}->{local}) {
        if (! $self->_genNagiosLocal($CONFIG, $self->{VO})) {
            close ($CONFIG);
            return;
        }
    }
    if ($self->{PROBES_TYPE_FLAG}->{remote}) {
        if (! $self->_genNagiosRemoteInstances($CONFIG) ) {
            close ($CONFIG);
            return;
        }
    }
    
    close ($CONFIG);

    1;
}

#############################
##
##  Contacts (contacts.cfg)
##
#############################

sub _genContactGroup {
    my $self = shift;
    my $CONFIG = shift;
    my $group = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{CONTACTS_CONTACTGROUP})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{CONTACTS_CONTACTGROUP}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<groupname>/$group/mg;
    print $CONFIG $line;

    1;
}

sub _genContact {
    my $self = shift;
    my $CONFIG = shift;
    my $contact = shift;
    my $email = shift;
    my $group = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{CONTACTS_CONTACT})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{CONTACTS_CONTACT}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<contactemail>/$email/mg;
    $line =~ s/<contactname>/$contact/mg;
    $line =~ s/<contactgroup>/$group/mg;
    print $CONFIG $line;

    1;
}

sub _genContacts {
	my $self = shift;
	my $vo = shift;
    my $CONFIG;
    my $TEMPL;
    my $line;
    my $contact;
    my $group;
    my $contacts;
    my $sitename = $self->{SITEDB}->siteName();

    if (!open ($CONFIG, ">" . $CONFIGURATION_OUTPUT->{CONTACTS})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_OUTPUT->{CONTACTS}."!");
        return;
    }

    print $CONFIG $self->{OUTPUT_HEADER};

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{CONTACTS})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{CONTACTS}."!");
        close ($CONFIG);
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<adminemail>/$self->{NAGIOS_ADMIN}/mg;
    print $CONFIG $line;

    # generate site level contact group
    $group = "$sitename-site";
    if (!$self->_genContactGroup($CONFIG, $group)) {
        close ($CONFIG);
        return;
    }

    my $check;
    foreach $contact ($self->{SITEDB}->getContacts)
    {
        if (!$self->_genContact($CONFIG, "$sitename-$contact", $contact, $group)) {
            close ($CONFIG);
            return;
        }
        $check = 1;
    }
    if (!$check) {
        $contact = 'root@localhost';
        if (!$self->_genContact($CONFIG, "$sitename-$contact", $contact, $group)) {
            close ($CONFIG);
            return;
        }
    }

    # generate host level contact group
    foreach my $host ($self->{SITEDB}->getRealHosts) {
        if ( $self->{SITEDB}->hasVO($host, $vo) &&  $self->{SITEDB}->hasContacts($host) ) {
            $group = "$sitename-$host";
            if (!$self->_genContactGroup($CONFIG, $group)) {
                close ($CONFIG);
                return;
            }

            foreach $contact ($self->{SITEDB}->getHostContacts($host)) {
                if (!exists $contacts->{$contact}) {
                    $contacts->{$contact} = $group;
                } else {
                    $contacts->{$contact} .= ",$group";
                }
            }
        }
    }

    # generate host contacts
    foreach $contact (keys %$contacts) {
        if (!$self->_genContact($CONFIG, "$sitename-$contact", $contact, $contacts->{$contact})) {
            close ($CONFIG);
            return;
        }
    }

    close ($CONFIG);

    1;
}

#############################
##
##  NRPE (wlcg_nrpe.cfg)
##    - configuration file for NRPE UI
##
#############################

sub _genNrpeLocalCommands {
    my $self = shift;
    my $CONFIG = shift;
    my $vo = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NRPE_LOCALCOMMANDS})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NRPE_LOCALCOMMANDS}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<VONAME>/$vo/mg;
    $line =~ s/<MYPROXY_SERVER>/$self->{MYPROXY_SERVER}/mg;
    $line =~ s/<NRPE_WLCG_PROBES_DIR>/$self->{NRPE_WLCG_PROBES_DIR}/mg;
    $line =~ s/<NRPE_PROXY_FILE>/$self->{NRPE_PROXY_FILE}/mg;
    $line =~ s/<NRPE_MYPROXY_NAME>/$self->{NRPE_MYPROXY_NAME}/mg;
    $line =~ s/<NRPE_PASSPHRASE_FILE>/$self->{NRPE_PASSPHRASE_FILE}/mg;
    print $CONFIG $line;

    1;
}

sub _genNrpeHeader {
	my $self = shift;
	my $vo = shift;
    my $CONFIG;
    my $line;

    if (!open ($CONFIG, ">" . $NRPE_CONFIGURATION_OUTPUT->{NRPE})) {
        $self->error("Cannot open configuration file ".$NRPE_CONFIGURATION_OUTPUT->{NRPE}."!");
        return;
    }

    print $CONFIG $self->{OUTPUT_HEADER};

    if (!$self->_genNrpeLocalCommands($CONFIG, $vo)) {
        close ($CONFIG);
        return;
    }

    return $CONFIG;
}

#############################
##
##  Hosts (hosts.cfg)
##
#############################

sub _getHostCheckType {
	my $self = shift;
	my $hostname = shift || return;
	my $checkType;

    if ( $self->checkPing($self->{SITEDB}->hostAddress($hostname)) ) {
        $checkType = "ncg_check_host_alive";
    } else {
        if ($self->{SITEDB}->hasService($hostname,"CE") || $self->{SITEDB}->hasService($hostname,"gLite-CE")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "GRAM_PORT");
        } elsif ($self->{SITEDB}->hasService($hostname,"SRM")) {
            if($self->{SITEDB}->hasMetricSet($hostname,"SRM2")) {
                $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "SRM2_PORT");
            } else {
                $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "SRM1_PORT");
            }
        } elsif ($self->{SITEDB}->hasService($hostname,"SE")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "SE_PORT");
        } elsif ($self->{SITEDB}->hasService($hostname,"Site-BDII") || $self->{SITEDB}->hasService($hostname,"BDII")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "BDII_PORT");
        } elsif ($self->{SITEDB}->hasService($hostname,"RB")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "RB_PORT");
        } elsif ($self->{SITEDB}->hasService($hostname,"WMS")){
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "WMS_PORT");
        } elsif ($self->{SITEDB}->hasService($hostname,"LFC")) {
            $checkType = "ncg_check_tcp!5010";
        } elsif ($self->{SITEDB}->hasService($hostname,"PROX")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "MYPROXY_PORT");
        } elsif ($self->{SITEDB}->hasService($hostname,"VOBOX")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "VOBOX_PORT");
        } elsif ($self->{SITEDB}->hasService($hostname,"MON") || $self->{SITEDB}->hasService($hostname,"VOMS") ||
		         $self->{SITEDB}->hasService($hostname,"gLite-FTS-WS")) {
            $checkType = "ncg_check_tcp!8443";
        }
    }

	return $checkType;
}

sub _getLBHostCheckType {
	my $self = shift;
	my $hostname = shift || return;
	my $lbaddress = shift || return;
	my $checkType;

    if ( $self->checkPing($lbaddress) ) {
        $checkType = "ncg_check_host_alive";
    } else {
        if ($self->{SITEDB}->hasRealService($hostname,"CE") || $self->{SITEDB}->hasRealService($hostname,"gLite-CE")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "GRAM_PORT");
        } elsif ($self->{SITEDB}->hasRealService($hostname,"SRM")) {
            if($self->{SITEDB}->hasRealMetricSet($hostname,"SRM2")) {
                $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "SRM2_PORT");
            } else {
                $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "SRM1_PORT");
            }
        } elsif ($self->{SITEDB}->hasRealService($hostname,"SE")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "SE_PORT");
        } elsif ($self->{SITEDB}->hasRealService($hostname,"Site-BDII") || $self->{SITEDB}->hasRealService($hostname,"BDII")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "BDII_PORT");
        } elsif ($self->{SITEDB}->hasRealService($hostname,"RB") ) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "RB_PORT");
        } elsif ($self->{SITEDB}->hasRealService($hostname,"WMS")){
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "WMS_PORT");
        } elsif ($self->{SITEDB}->hasRealService($hostname,"LFC")) {
            $checkType = "ncg_check_tcp!5010";
        } elsif ($self->{SITEDB}->hasRealService($hostname,"PROX")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "MYPROXY_PORT");
        } elsif ($self->{SITEDB}->hasRealService($hostname,"VOBOX")) {
            $checkType = "ncg_check_tcp!".$self->{SITEDB}->hostAttribute($hostname, "VOBOX_PORT");
        } elsif ($self->{SITEDB}->hasRealService($hostname,"MON") || $self->{SITEDB}->hasRealService($hostname,"VOMS") ||
		         $self->{SITEDB}->hasRealService($hostname,"gLite-FTS-WS")) {
            $checkType = "ncg_check_tcp!8443";
        }
    }

	return $checkType;
}

sub _genHostGroup {
    my $self = shift;
    my $CONFIG = shift;
    my $name = shift;
    my $fullname = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{HOSTS_HOSTGROUP})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{HOSTS_HOSTGROUP}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<name>/$name/mg;
    $line =~ s/<fullname>/$fullname/mg;
    print $CONFIG $line;
    1;
}

# ($hostname, $address, $checkType, $contactList, $hostgroup, $alias);
sub _genHost {
    my $self = shift;
    my $CONFIG = shift;
    my $hostname = shift;
    my $address = shift;
    my $checkType = shift;
    my $contactList = shift;
    my $hostgroup = shift;
    my $alias = shift;
    my $TEMPL;
    my $line;

    if ($hostgroup) {
        $hostgroup = "hostgroups                      $hostgroup";
    }

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{HOSTS_HOST})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_TEMPLATES->{HOSTS_HOST}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<alias>/$alias/mg;
    $line =~ s/<hostgroup>/$hostgroup/mg;
    $line =~ s/<hostname>/$hostname/mg;
    $line =~ s/<address>/$address/mg;
    $line =~ s/<checktype>/$checkType/mg;
    $line =~ s/<contactlist>/$contactList/mg;
    print $CONFIG $line;
    
    1;
}

sub _genNagiosHost {
    my $self = shift;
    my $CONFIG = shift;
    my $TEMPL;
    my $line;
    my $checkType;

    $self->_genHost($CONFIG,
                    $self->{NAGIOS_SERVER},
                    $self->{NAGIOS_SERVER_ADDRESS}, 
                    "ncg_check_host_alive",
                    "nagios-admins", "", "Nagios server");

}

sub _genMyProxyHost {
    my $self = shift;
    my $CONFIG = shift;
    my $TEMPL;
    my $line;
    my $checkType;

    if ( $self->checkPing($self->{MYPROXY_SERVER_ADDRESS}) ) {
        $checkType = "ncg_check_host_alive";
    } else {
        $checkType = "ncg_check_tcp!7512";
    }

    $self->_genHost($CONFIG,
                    $self->{MYPROXY_SERVER},
                    $self->{MYPROXY_SERVER_ADDRESS},
                    $checkType,
                    "nagios-admins", "", "MyProxy server");
}

sub _genNrpeHost {
    my $self = shift;
    my $CONFIG = shift;
    my $TEMPL;
    my $line;
    my $checkType;

    if ( $self->checkPing($self->{NRPE_UI_ADDRESS}) ) {
        $checkType = "ncg_check_host_alive";
    } else {
        $checkType = "ncg_check_nrpe";
    }

    $self->_genHost($CONFIG,
                    $self->{NRPE_UI},
                    $self->{NRPE_UI_ADDRESS},
                    $checkType,
                    "nagios-admins", "", "NRPE UI server");
}

sub _genHostsHeader {
	my $self = shift;
    my $CONFIG;
    my $TEMPL;
    my $line;
    my $hosts = {};

    if (!open ($CONFIG, ">" . $CONFIGURATION_OUTPUT->{HOSTS})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_OUTPUT->{HOSTS}."!");
        return;
    }

    print $CONFIG $self->{OUTPUT_HEADER};

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{HOSTS})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{HOSTS}."!");
        close ($CONFIG);
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    print $CONFIG $line;

    # Generate configuration for Nagios server
    if (! $self->{SITEDB}->hasHost($self->{NAGIOS_SERVER}) ) {
        if (!$self->_genNagiosHost($CONFIG)){
            close ($CONFIG);
            return;
        }
        $hosts->{$self->{NAGIOS_SERVER}} = 1;
    }

    # Generate configuration for MyProxy server
    if ($self->{PROBES_TYPE_FLAG}->{local} && ! exists $hosts->{$self->{MYPROXY_SERVER}} && ! $self->{SITEDB}->hasHost($self->{MYPROXY_SERVER}) ) {
        if (!$self->_genMyProxyHost($CONFIG)){
            close ($CONFIG);
            return;
        }
        $hosts->{$self->{MYPROXY_SERVER}} = 1;
    }

    # Generate configuration for NRPE UI server
    if ($self->{NRPE_UI} && $self->{PROBES_TYPE_FLAG}->{local} && ! exists $hosts->{$self->{NRPE_UI}} && ! $self->{SITEDB}->hasHost($self->{NRPE_UI}) ) {
        if (!$self->_genNrpeHost($CONFIG)){
            close ($CONFIG);
            return;
        }
    }

    return $CONFIG;
}

#############################
##
##  Services (service.cfg)
##
#############################

sub _getMetricOptionString {
    my $self = shift;
    my $host = shift;
    my $metric = shift;
    my $vo = shift;
    my $attributes = $self->{SITEDB}->metricAttributes($host, $metric);
    my $options = "";

    foreach my $attr (%$attributes) {
        my $value = $self->{SITEDB}->hostAttribute($host, $attr);

        if ($value && ref $value eq "HASH") {
            $value = $self->{SITEDB}->hostAttributeVO($host, $attr, $vo);
        }
        
        if ($attr eq 'VONAME') {
            $value = $vo;
        }
        
        if (!$value) {
            next;
        }
        
        $options .= $attributes->{$attr} . ' '  . $value . ' ';
    }
    
    return $options;
}

sub _genServiceGroup {
    my $self = shift;
    my $CONFIG = shift;
    my $name = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{SERVICES_SERVICEGROUP})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_TEMPLATES->{SERVICES_SERVICEGROUP}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<service>/$name/mg;
    print $CONFIG $line;

    1;
}

sub _genServiceDependency {
    my $self = shift;
    my $CONFIG = shift;
    my $hostname1 = shift;
    my $servicename1 = shift;
    my $hostname2 = shift;
    my $servicename2 = shift;
    my $TEMPL;
    my $line;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{SERVICES_SERVICEDEP})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_TEMPLATES->{SERVICES_SERVICEDEP}."!");
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    $line =~ s/<servicename1>/$servicename1/mg;
    $line =~ s/<servicename2>/$servicename2/mg;
    $line =~ s/<hostname1>/$hostname1/mg;
    $line =~ s/<hostname2>/$hostname2/mg;

    print $CONFIG $line;
    
    1;
}

sub _genWlcgService {
    my $self = shift;
    my $CONFIG = shift;
    my $line = shift;
    my $hostname = shift;
    my $metric = shift;
    my $probe = shift;
    my $contactlist = shift;
    my $servicegroup = shift;
    my $serviceuri = shift;
    my $user = shift;
    my $timeout = shift;
    my $interval = shift;
    my $retryInterval = shift;
    my $options = shift;
    my $docUrl = shift || "";

    if ($docUrl) {
        $docUrl = "action_url                      $docUrl";
    }

    $line =~ s/<hostname>/$hostname/mg;
    $line =~ s/<servicegroup>/$servicegroup/mg;
    $line =~ s/<contactlist>/$contactlist/mg;
    $line =~ s/<probe>/$probe/mg;
    $line =~ s/<metric>/$metric/mg;
    $line =~ s/<serviceuri>/$serviceuri/mg;
    $line =~ s/<timeout>/$timeout/mg;
    $line =~ s/<options>/$options/mg;
    $line =~ s/<interval>/$interval/mg;
    $line =~ s/<retryInterval>/$retryInterval/mg;
    $line =~ s/<USER_PROXY>/$user/mg;
    $line =~ s/<docurl>/$docUrl/mg;
    print $CONFIG $line;
    
    1;
}

sub _genWlcgServiceNrpe {
    my $self = shift;
    my $CONFIG = shift;
    my $NRPE_CONFIG = shift;
    my $line = shift;
    my $nrpeLine = shift;

    my $hostname = shift;
    my $metric = shift;
    my $probe = shift;
    my $contactlist = shift;
    my $servicegroup = shift;
    my $serviceuri = shift;
    my $timeout = shift;
    my $interval = shift;
    my $retryInterval = shift;
    my $options = shift;
    my $docUrl = shift || "";
    my $nrpecommand = $hostname . "_" . $metric;

    if ($docUrl) {
        $docUrl = "action_url                      $docUrl";
    }

    $line =~ s/<hostname>/$hostname/mg;
    $line =~ s/<servicegroup>/$servicegroup/mg;
    $line =~ s/<metric>/$metric/mg;
    $line =~ s/<contactlist>/$contactlist/mg;
    $line =~ s/<nrpecommand>/$nrpecommand/mg;
    $line =~ s/<timeout>/$timeout/mg;
    $line =~ s/<interval>/$interval/mg;
    $line =~ s/<retryInterval>/$retryInterval/mg;
    $line =~ s/<docurl>/$docUrl/mg;
    $line =~ s/<NRPE_UI>/$self->{NRPE_UI}/mg;
    print $CONFIG $line;

    $nrpeLine =~ s/<NRPE_WLCG_PROBES_DIR>/$self->{NRPE_WLCG_PROBES_DIR}/mg;
    $nrpeLine =~ s/<NRPE_WLCG_PLUGINS_DIR>/$self->{NRPE_WLCG_PLUGINS_DIR}/mg;
    $nrpeLine =~ s/<NRPE_PROXY_FILE>/$self->{NRPE_PROXY_FILE}/mg;
    $nrpeLine =~ s/<probe>/$probe/mg;
    $nrpeLine =~ s/<metric>/$metric/mg;
    $nrpeLine =~ s/<serviceuri>/$serviceuri/mg;
    $nrpeLine =~ s/<timeout>/$timeout/mg;
    $nrpeLine =~ s/<options>/$options/mg;
    $nrpeLine =~ s/<nrpecommand>/$nrpecommand/mg;
    print $NRPE_CONFIG $nrpeLine;

    1;
}

sub _genNativeService {
    my $self = shift;
    my $CONFIG = shift;
    my $line = shift;
    my $hostname = shift;
    my $metric = shift;
    my $probe = shift;
    my $contactlist = shift;
    my $servicegroup = shift;
    my $timeout = shift;
    my $interval = shift;
    my $retryInterval = shift;
    my $options = shift;    

    $line =~ s/<hostname>/$hostname/mg;
    $line =~ s/<servicegroup>/$servicegroup/mg;
    $line =~ s/<metric>/$metric/mg;
    $line =~ s/<contactlist>/$contactlist/mg;
    $line =~ s/<probe>/$probe/mg;
    $line =~ s/<options>/$options/mg;
    $line =~ s/<timeout>/$timeout/mg;
    $line =~ s/<interval>/$interval/mg;
    $line =~ s/<retryInterval>/$retryInterval/mg;

    print $CONFIG $line;
    
    1;
}

# ($host, $servicename, $remoteService, $contactgroup, $servicegroup, $url)
sub _genRemoteService {
    my $self = shift;
    my $CONFIG = shift;
    my $line = shift;
    my $host = shift;
    my $servicename = shift;
    my $remoteService = shift;
    my $contactgroup = shift;
    my $servicegroup = shift;
    my $url = shift || "";
    my $docurl = shift || "";

    if ($url) {
        $url = "notes_url                       $url";
    }

    if ($docurl) {
        $docurl = "action_url                      $docurl";
    }

    $line =~ s/<hostname>/$host/mg;
    $line =~ s/<servicename>/$servicename/mg;
    $line =~ s/<servicegroup>/$servicegroup/mg;
    $line =~ s/<contactlist>/$contactgroup/mg;
    $line =~ s/<url>/$url/mg;
    $line =~ s/<docurl>/$docurl/mg;
    $line =~ s/<NAGIOS_SERVER>/$self->{NAGIOS_SERVER}/mg;
    print $CONFIG $line;
    
    1;
}

sub _getLocalServiceTemplates {
    my $self = shift;
    my $templates = shift;
    my $TEMPL;

    if ($self->{NRPE_UI}) {
        if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{SERVICES_WLCGNRPE})) {
            $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{SERVICES_WLCGNRPE}."!");
            return;
        }
        $templates->{WLCGNRPE} = join ("", <$TEMPL>);
        close ($TEMPL);
        if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{NRPE_WLCGCOMMANDS})) {
            $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{NRPE_WLCGCOMMANDS}."!");
            return;
        }
        $templates->{NRPE_WLCGCOMMANDS} = join ("", <$TEMPL>);
        close ($TEMPL);
    } else {
        if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{SERVICES_WLCG})) {
            $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{SERVICES_WLCG}."!");
            return;
        }
        $templates->{WLCG} = join ("", <$TEMPL>);
        close ($TEMPL);
    }

    1;
}

sub _getNativeServiceTemplates {
    my $self = shift;
    my $templates = shift;
    my $TEMPL;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{SERVICES_NATIVE})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{SERVICES_NATIVE}."!");
        return;
    }
    $templates->{NATIVE} = join ("", <$TEMPL>);
    close ($TEMPL);
    
    1;
}

sub _getRemoteServiceTemplates {
    my $self = shift;
    my $templates = shift;
    my $TEMPL;

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{SERVICES_SAM})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_TEMPLATES->{SERVICES_SAM}."!");
        return;
    }
    $templates->{SAM} = join ("", <$TEMPL>);
    close ($TEMPL);

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{SERVICES_NPM})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_TEMPLATES->{SERVICES_NPM}."!");
        return;
    }
    $templates->{NPM} = join ("", <$TEMPL>);
    close ($TEMPL);
    
    1;
}

sub _genServices {
    my $self = shift;
    my $CONFIG = shift;
    my $host = shift;
    my $contactgroup = shift;
    my $servicegroups = shift;
    my $NRPE_CONFIG = shift;
    my $sitename = $self->{SITEDB}->siteName();
    my $servicegroup;

    if ($self->{PROBES_TYPE_FLAG}->{local}) {
        my $user = "USER" . ($self->{USER_OFFSET}+4);
        my $templates = {};
        if (!$self->_getLocalServiceTemplates ($templates)) {
            return;
        }

        foreach my $metric ($self->{SITEDB}->getLocalMetrics($host)) {
            $self->verbose ("    metric: $metric");
            my $options = $self->_getMetricOptionString($host, $metric, $self->{VO});
            my $metricSet = $self->{SITEDB}->metricMetricSet($host, $metric);
            my $config = $self->{SITEDB}->metricConfig($host, $metric);
            $servicegroups->{local} = 1;
            $servicegroups->{$metricSet} = 1;

            if ($self->{NRPE_UI}) {
                if (! $self->_genWlcgServiceNrpe($CONFIG,
                                            $NRPE_CONFIG,
                                            $templates->{WLCGNRPE},
                                            $templates->{NRPE_WLCGCOMMANDS},
                                            $host,
                                            $metric,
                                            $self->{SITEDB}->metricProbe($host, $metric),
                                            $contactgroup,
                                            "local, $metricSet",
                                            $self->{SITEDB}->metricSetServiceURI($host, $metricSet),
                                            $config->{timeout},
                                            $config->{interval},
                                            $config->{retryInterval},
                                            $options,
                                            $self->{SITEDB}->metricDocUrl($host, $metric)) ) {
                    return;
                }
            } else {
                if (! $self->_genWlcgService($CONFIG,
                                            $templates->{WLCG},
                                            $host,
                                            $metric,
                                            $self->{SITEDB}->metricProbe($host, $metric),
                                            $contactgroup,
                                            "local, $metricSet",
                                            $self->{SITEDB}->metricSetServiceURI($host, $metricSet),
                                            $user,
                                            $config->{timeout},
                                            $config->{interval},
                                            $config->{retryInterval},
                                            $options,
                                            $self->{SITEDB}->metricDocUrl($host, $metric)) ) {
                    return;
                }
            }

            # generate dependencies
            my $deps = $self->{SITEDB}->metricDependencies($host, $metric);
            my $host2;
            my $gridProxyServer;
            if ($self->{NRPE_UI}) {
                $gridProxyServer = $self->{NRPE_UI};
            } else {
                $gridProxyServer = $self->{NAGIOS_SERVER};
            }
            foreach my $dep (keys %$deps) {
                if ($dep eq "hr.srce.GridProxy-Valid") {
                    $host2 = $gridProxyServer;
                } else {
                    $host2 = $host;
                }

                if (! $self->_genServiceDependency($CONFIG, $host, $metric, $host2, $dep) ) {
                    return;
                }
            }
        }
    }

    if ($self->{PROBES_TYPE_FLAG}->{native}) {
        my $templates = {};
        if (!$self->_getNativeServiceTemplates ($templates)) {
            return;
        }

        foreach my $metric ($self->{SITEDB}->getNativeMetrics($host, 'Nagios')) {
            $self->verbose ("    metric: $metric");
            my $options = $self->_getMetricOptionString($host, $metric, $self->{VO});
            my $metricSet = $self->{SITEDB}->metricMetricSet($host, $metric);
            my $config = $self->{SITEDB}->metricConfig($host, $metric);
            $servicegroups->{native} = 1;
            $servicegroups->{$metricSet} = 1;

            if (! $self->_genNativeService($CONFIG,
                                        $templates->{NATIVE},
                                        $host,
                                        $metric,
                                        $self->{SITEDB}->metricProbe($host, $metric),
                                        $contactgroup,
                                        "native, $metricSet",
                                        $config->{timeout},
                                        $config->{interval},
                                        $config->{retryInterval},
                                        $options )) {
                return;
            }

            # generate dependencies
            my $deps = $self->{SITEDB}->metricDependencies($host, $metric);
            foreach my $dep (keys %$deps) {
                if (! $self->_genServiceDependency($CONFIG, $host, $metric, $host, $dep) ) {
                    return;
                }
            }
        }
    }


    if ($self->{PROBES_TYPE_FLAG}->{remote}) {
        my $templates = {};
        if (!$self->_getRemoteServiceTemplates ($templates)) {
            return;
        }

        foreach my $metric ($self->{SITEDB}->getRemoteMetrics($host)) {
            $self->verbose ("    metric: $metric");
            my $remoteService = $self->{SITEDB}->metricRemoteService($host, $metric);
            $servicegroups->{remote} = 1;
            $servicegroups->{$remoteService} = 1;

            # ($host, $servicename, $remoteService, $contactgroup, $servicegroup, $url)
            if (! $self->_genRemoteService($CONFIG,
                                         $templates->{$remoteService},
                                         $host,
                                         $metric,
                                         $remoteService,
                                         $contactgroup,
                                         "remote, $remoteService",
                                         $self->{SITEDB}->metricUrl($host, $metric),
                                         $self->{SITEDB}->metricDocUrl($host, $metric)) ) {
                return;
            }
        }
    }

    1;
}

sub _genLBServices {
    my $self = shift;
    my $CONFIG = shift;
    my $lbnode = shift;
    my $host = shift;
    my $contactgroup = shift;
    my $servicegroups = shift;
    my $NRPE_CONFIG = shift;

    my $sitename = $self->{SITEDB}->siteName();
    my $servicegroup;

    if ($self->{PROBES_TYPE_FLAG}->{local}) {
        my $user = "USER" . ($self->{USER_OFFSET}+4);
        my $templates = {};
        if (!$self->_getLocalServiceTemplates ($templates)) {
            return;
        }

        foreach my $metric ($self->{SITEDB}->getRealLocalMetrics($host)) {
            $self->verbose ("    metric: $metric");
            my $options = $self->_getMetricOptionString($host, $metric, $self->{VO});
            my $metricSet = $self->{SITEDB}->metricMetricSet($host, $metric);
            my $config = $self->{SITEDB}->metricConfig($host, $metric);
            $servicegroups->{local} = 1;
            $servicegroups->{$metricSet} = 1;

            if ($self->{NRPE_UI}) {
                if (! $self->_genWlcgServiceNrpe($CONFIG,
                                            $NRPE_CONFIG,
                                            $templates->{WLCGNRPE},
                                            $templates->{NRPE_WLCGCOMMANDS},
                                            $lbnode,
                                            $metric,
                                            $self->{SITEDB}->metricProbe($host, $metric),
                                            $contactgroup,
                                            "local, $metricSet",
                                            $self->{SITEDB}->metricSetServiceURILBNode($lbnode, $host, $metricSet),
                                            $config->{timeout},
                                            $config->{interval},
                                            $config->{retryInterval},
                                            $options,
                                            $self->{SITEDB}->metricDocUrl($host, $metric)) ) {
                    return;
                }
            } else {
                if (! $self->_genWlcgService($CONFIG,
                                            $templates->{WLCG},
                                            $lbnode,
                                            $metric,
                                            $self->{SITEDB}->metricProbe($host, $metric),
                                            $contactgroup,
                                            "local, $metricSet",
                                            $self->{SITEDB}->metricSetServiceURILBNode($lbnode, $host, $metricSet),
                                            $user,
                                            $config->{timeout},
                                            $config->{interval},
                                            $config->{retryInterval},
                                            $options,
                                            $self->{SITEDB}->metricDocUrl($host, $metric)) ) {
                    return;
                }
            }

            # generate dependencies
            my $deps = $self->{SITEDB}->metricDependencies($host, $metric);
            my $host2;
            my $gridProxyServer;
            if ($self->{NRPE_UI}) {
                $gridProxyServer = $self->{NRPE_UI};
            } else {
                $gridProxyServer = $self->{NAGIOS_SERVER};
            }
            foreach my $dep (keys %$deps) {
                if ($dep eq "hr.srce.GridProxy-Valid") {
                    $host2 = $gridProxyServer;
                } else {
                    $host2 = $lbnode;
                }

                if (! $self->_genServiceDependency($CONFIG, $lbnode, $metric, $host2, $dep) ) {
                    return;
                }
            }
        }
    }
    
    if ($self->{PROBES_TYPE_FLAG}->{native}) {
        my $templates = {};
        if (!$self->_getNativeServiceTemplates ($templates)) {
            return;
        }

        foreach my $metric ($self->{SITEDB}->getRealNativeMetrics($host, 'Nagios')) {
            $self->verbose ("    metric: $metric");
            my $options = $self->_getMetricOptionString($host, $metric, $self->{VO});
            my $metricSet = $self->{SITEDB}->metricMetricSet($host, $metric);
            my $config = $self->{SITEDB}->metricConfig($host, $metric);
            $servicegroups->{native} = 1;
            $servicegroups->{$metricSet} = 1;

            if (! $self->_genNativeService($CONFIG,
                                        $templates->{NATIVE},
                                        $lbnode,
                                        $metric,
                                        $self->{SITEDB}->metricProbe($host, $metric),
                                        $contactgroup,
                                        "native, $metricSet",
                                        $config->{timeout},
                                        $config->{interval},
                                        $config->{retryInterval},
                                        $options )) {
                return;
            }

            # generate dependencies
            my $deps = $self->{SITEDB}->metricDependencies($host, $metric);
            foreach my $dep (keys %$deps) {
                if (! $self->_genServiceDependency($CONFIG, $lbnode, $metric, $lbnode, $dep) ) {
                    return;
                }
            }
        }
    }
    
    1;
}


sub _genServicesHeader {
	my $self = shift;
    my $CONFIG;
    my $TEMPL;
    my $line;

    if (!open ($CONFIG, ">" . $CONFIGURATION_OUTPUT->{SERVICES})) {
        $self->error("Cannot open configuration file ".$CONFIGURATION_OUTPUT->{SERVICES}."!");
        return;
    }

    print $CONFIG $self->{OUTPUT_HEADER};

    if (!open ($TEMPL, $CONFIGURATION_TEMPLATES->{SERVICES})) {
        $self->error("Cannot open template file ".$CONFIGURATION_TEMPLATES->{SERVICES}."!");
        close ($CONFIG);
        return;
    }
    $line = join ("", <$TEMPL>);
    close ($TEMPL);
    print $CONFIG $line;

    return $CONFIG;
}

#############################
##
##  ConfigGen getConfig implementation
##
#############################

sub _loadHeaderText
{
    my $self = shift;
    my $username = getlogin();
    my $time = time2str("%Y-%m-%dT%H:%M:%SZ",time(),"UTC");
    my $settings = "";

    foreach my $attr (keys %$self) {
        $settings .= "#     ".$attr.": ".$self->{$attr}."\n" if (!ref($self->{$attr}));
    }
    chop $settings;

    $self->{OUTPUT_HEADER} =
"#
# Automatically generated by NCG::ConfigGen::Nagios.pm by $username at $time
#
# Settings:
$settings
#

";
}

sub _getContactGroups {
	my $self = shift;
    my $host = shift || return;
    my $sitename = $self->{SITEDB}->siteName();
    my $contactgroup = "$sitename-site";

    if ($self->{SITEDB}->hasContacts($host)) {
        $contactgroup .= ", $sitename-$host";
    }

    if ((defined $self->{MYPROXY_SERVER} && $host eq $self->{MYPROXY_SERVER}) ||
        $host eq $self->{NAGIOS_SERVER} ||
        (defined $self->{WLCG_UI} && $host eq $self->{WLCG_UI})) {
        $contactgroup .= ", nagios-admins";
    }

    $contactgroup;
}

sub _getHostGroups {
	my $self = shift;
    my $host = shift || return;
    my $hostgroups = shift;
    my $hostgroup;
    my $sitename = $self->{SITEDB}->siteName();

    # each host belongs to site hostgroup
    # (this hostgroup is useful on regional level Nagios)
    $hostgroup = "site-$sitename";
    $hostgroups->{"site-$sitename"} = "Site $sitename";

    # for each service group is generated
    foreach my $service ($self->{SITEDB}->getServices($host)) {
        $hostgroup .= ", node-$service";
        $hostgroups->{"node-$service"} = "$service nodes";
    }
    
    # if host is an alias, generate group of aliases
    my $alias;
    if ($alias = $self->{SITEDB}->hostAlias($host)) {
        $hostgroup .= ", alias-$alias";
        $hostgroups->{"alias-$alias"} = "Aliases of host $alias";
    }

    $hostgroup;
}

# difference between _getHostGroups:
#  - getRealServices is used for node-* groups
#  - lbnode-* is used instead of alias-*
sub _getLBHostGroups {
	my $self = shift;
    my $host = shift || return;
    my $hostgroups = shift;
    my $hostgroup;
    my $sitename = $self->{SITEDB}->siteName();

    # each host belongs to site hostgroup
    # (this hostgroup is useful on regional level Nagios)
    $hostgroup = "site-$sitename";
    $hostgroups->{"site-$sitename"} = "Site $sitename";

    # for each service group is generated
    foreach my $service ($self->{SITEDB}->getRealServices($host)) {
        $hostgroup .= ", node-$service";
        $hostgroups->{"node-$service"} = "$service nodes";
    }

    $hostgroup .= ", lbnode-$host";
    $hostgroups->{"lbnode-$host"} = "Load balanced nodes of host $host";

    $hostgroup;
}

sub _closeFDs {
	my $self = shift;
    my $HOST_CONFIG = shift;
    my $SERVICE_CONFIG = shift;
    my $NRPE_CONFIG = shift;

    close($HOST_CONFIG) if ($self->{INCLUDE_HOSTS});
    close($SERVICE_CONFIG);
    close($NRPE_CONFIG) if ($self->{NRPE_UI});
}

sub getData {
	my $self = shift;
    my $HOST_CONFIG;
    my $SERVICE_CONFIG;
    my $NRPE_CONFIG;
    my $sitename = $self->{SITEDB}->siteName();
    my $servicegroups = {};
    my $hostgroups = {};

    $self->verbose("Generating configuration:");

    # Load header text which is added to all output files
    $self->_loadHeaderText();

    # Generate commands
    $self->_genCommands();

    # Generate Nagios local stuff
    #  - in case of native probes only this is not required
    if ($self->{PROBES_TYPE_FLAG}->{local} || $self->{PROBES_TYPE_FLAG}->{remote}) {
        $self->_genNagios($self->{VO});
    }

    # In case of remote UI generate NRPE config
    if ($self->{NRPE_UI}) {
        $NRPE_CONFIG = $self->_genNrpeHeader($self->{VO});
        if (!$NRPE_CONFIG) {
            return;
        }
    }

    # Open hosts.cfg file descriptor
    if ($self->{INCLUDE_HOSTS}) {
        $HOST_CONFIG = $self->_genHostsHeader();
        if (!$HOST_CONFIG) {
            return;
        }
    }

    # Open services.cfg file descriptor
    $SERVICE_CONFIG = $self->_genServicesHeader();
    if (!$SERVICE_CONFIG) {
        close ($HOST_CONFIG) if ($self->{INCLUDE_HOSTS});
        close ($NRPE_CONFIG) if ($self->{NRPE_UI});
        return;
    }

    # Iterate through hosts
    foreach my $host ($self->{SITEDB}->getHosts) {
        $self->verbose("  host: $host");
        if ($self->{SITEDB}->hasVO($host, $self->{VO})) {
            my $contactgroup = $self->_getContactGroups ($host);
            my $hostgroup = $self->_getHostGroups ($host, $hostgroups);
            my $checktype = $self->_getHostCheckType($host) || next;
            
            if ($self->{INCLUDE_HOSTS}) {
                if (! $self->_genHost ($HOST_CONFIG,
                                 $host,
                                 $self->{SITEDB}->hostAddress($host),
                                 $checktype,
                                 $contactgroup,
                                 $hostgroup,
                                 $host) ) {
                    $self->_closeFDs($HOST_CONFIG, $SERVICE_CONFIG, $NRPE_CONFIG);
                    return;
                }
            }

            if (! $self->_genServices ($SERVICE_CONFIG, $host, $contactgroup, $servicegroups, $NRPE_CONFIG)) {
                $self->_closeFDs($HOST_CONFIG, $SERVICE_CONFIG, $NRPE_CONFIG);
                return;
            }
        }
    }

    # Iterate through hosts which have LB nodes
    foreach my $host ($self->{SITEDB}->getRealHosts) {
        if ($self->{SITEDB}->hasLBNodes($host) && $self->{SITEDB}->hasVO($host, $self->{VO})) {
            my $contactgroup = $self->_getContactGroups ($host);
            my $hostgroup = $self->_getLBHostGroups ($host, $hostgroups);
            
            # Iterate through LB nodes
            foreach my $lbnode ($self->{SITEDB}->getLBNodes($host)) {
                if ($self->{INCLUDE_HOSTS}) {
                    my $lbaddress = $self->{SITEDB}->LBNodeAddress($host, $lbnode);
                    my $checktype = $self->_getLBHostCheckType($host,$lbaddress) || next;
                    if (! $self->_genHost ($HOST_CONFIG,
                                     $lbnode,
                                     $lbaddress,
                                     $checktype,
                                     $contactgroup,
                                     $hostgroup,
                                     $lbnode) ) {
                        $self->_closeFDs($HOST_CONFIG, $SERVICE_CONFIG, $NRPE_CONFIG);
                        return;
                    }
                }

                if (! $self->_genLBServices ($SERVICE_CONFIG, $lbnode, $host, $contactgroup, $servicegroups, $NRPE_CONFIG)) {
                    $self->_closeFDs($HOST_CONFIG, $SERVICE_CONFIG, $NRPE_CONFIG);
                    return;
                }
            }
        }
    }

    # Generate hostgroups
    if ($self->{INCLUDE_HOSTS}) {
        foreach my $group (keys %$hostgroups) {
            if (! $self->_genHostGroup ($HOST_CONFIG, $group, $hostgroups->{$group}) ) {
                $self->_closeFDs($HOST_CONFIG, $SERVICE_CONFIG, $NRPE_CONFIG);
                return;
            }
        }
    }

    # Generate servicegroups
    foreach my $sgroup (keys %$servicegroups) {
        if (! $self->_genServiceGroup ($SERVICE_CONFIG, $sgroup) ) {
            $self->_closeFDs($HOST_CONFIG, $SERVICE_CONFIG, $NRPE_CONFIG);
            return;
        }
    }

    # Generate contacts
    if (! $self->_genContacts($self->{VO}) ) {
        $self->_closeFDs($HOST_CONFIG, $SERVICE_CONFIG, $NRPE_CONFIG);
        return;
    }

    $self->_closeFDs($HOST_CONFIG, $SERVICE_CONFIG, $NRPE_CONFIG);
    
    1;
}

=head1 NAME

NCG::ConfigGen::Nagios

=head1 DESCRIPTION

The NCG::ConfigGen::Nagios module extends NCG::ConfigGen module.
Module generates configuration for Nagios monitoring system.

Module has following requirements on native Nagios configuration:
- resource variable $USER1$ should point to location of
Nagios plugins. If this is not the case, please modify 
template commands.template.
- following Nagios plugins are used:
  - check_tcp, check_ping
  - native plugins included in NCG::LocalMetrics modules
  (currently: check_tcp, check_ftp, check_ldap)

Generated configuration should not interfere with the existing
Nagios configuration. Following properties can be used to tune
configuration in case when existing Nagios server is used:
- INCLUDE_HOSTS - defines if hosts configuration is generated
- USER_OFFSET - defines first index of resource variable which
should be used.

=head1 SYNOPSIS

  use NCG::ConfigGen::Nagios;

  my $ncg = NCG::ConfigGen::Nagios->new( { SITEDB=> $sitedb,
                                           TEMPLATES_DIR=> '/opt/lcg/share/templates',
                                           OUTPUT_DIR=>'/etc/nagios/wlcg.d' } );

  $ncg->getData();

=cut

=head1 METHODS

=over

=item C<new>

  $ncg = NCG::ConfigGen::Nagios->new( $options );

Creates new NCG::ConfigGen::Nagios instance. Argument $options is hash
reference that can contain following elements:
  GLITE_VERSION - which version of Glite UI the tests will run on.
  (default: UNKNOWN)

  INCLUDE_HOSTS - if true hosts definitions (hosts.cfg) will be
  generated.
  (default: true)
  
  MYPROXY_SERVER - MyProxy server where user credentials are stored.
  (default: )

  NAGIOS_ADMIN - List of email addresses set for Nagios internal checks:
            GridProxy-Get, GridProxy-Valid, MyProxy-ProxyLifetime,
            CADist-Version (on Nagios server).
  (default: root@localhost)
  
  NRPE_OUTPUT_DIR - path where to write NRPE configuration for remote UI.
  This should not be the same as OUTPUT_DIR because Nagios will report
  error when parsing NRPE configuration file. If these are the same, 
  generated wlcg.nrpe.cfg should be manually removed before starting
  Nagios.
  (default: /etc/nagios)

  NRPE_UI - set to address of remote UI server which is used to run
  local probes. (Not implemented yet)
  (default: )

  NRPE_WLCG_PLUGINS_DIR - location of check_wlcg Nagios plugin which
  is used for running WLCG probes.
  This option is required only if NRPE_UI is defined, because NRPE does
  not support macros and resource configuration. Commands for NRPE need
  to have full paths of executables defined.
  (default: /usr/lib/nagios/plugins/wlcg)

  NRPE_WLCG_PROBES_DIR - location of WLCG probes on NRPE server.
  This option is required only if NRPE_UI is defined, because NRPE does
  not support macros and resource configuration.
  (default: /opt/lcg/share/grid-monitoring/plugins)

  NRPE_PROXY_FILE - location of proxy credential on NRPE server.
  This option is required only if NRPE_UI is defined, because NRPE does
  not support macros and resource configuration.
  (default: /etc/nagios/globus/userproxy.pem)

  NRPE_PASSPHRASE_FILE - location of file with passphrase for MyProxy
  credential on NRPE server.
  This option is required only if NRPE_UI is defined, because NRPE does
  not support macros and resource configuration.
  (default: /etc/nagios/globus/proxy_passphrase)

  NRPE_MYPROXY_NAME - name of MyProxy credential which is used to
  regenerate proxy.
  This option is required only if NRPE_UI is defined, because NRPE does
  not support macros and resource configuration.
  (default: NagiosRefresh)

  OUTPUT_DIR - path where to write configuration.
  (default: /etc/nagios/wlcg.d)
  
  PROBES_TYPE - which probes to include in configuration.
                Possible values:
                    local  - only locally executed probes are included.
                             Nagios won't pull results from external
                             monitoring systems (SAM).
                    remote - only remotely executed probes are included.
                             Nagios won't run any active probes. MyProxy
                             settings are not required in this case.
                    native - only native Nagios probes are included.
                             MyProxy settings are not required in this case.
                    all    - all probe types are included.
  (default: all)

  SAM_INCLUDE_ALL - if defined gather_sam will be configured to fetch all 
  SAM tests. Otherwise only critical tests are gathered.
  This option should have the same value as the equivalent one in 
  module NCG::RemoteMetrics::SAM
  (default: )

  SAM_ROOT_URL - root URL used for SAM DB query interface
  This option should have the same value as the equivalent one in
  module NCG::RemoteMetrics::SAM
  (default: http://lcg-sam.cern.ch:8080/same-pi/)

  SAM_VOS - list of VOs for which SAM results will be gathered.
  If not defined all supported VOS on site are used. List of
  supported VOs is fetched from SAM PI.
  Format: VO1[,VO]...
  This option should have the same value as the equivalent one in
  module NCG::RemoteMetrics::SAM
  (default: )

  TEMPLATES_DIR - path to templates directory.
  (default: /opt/lcg/share/grid-monitoring/config-gen/nagios)
  
  USER_OFFSET - defines first index of USERX variable which can be used
  by NCG.

Constructor checks if all templates are present and if output directory
is writeable.

=back

=head1 SEE ALSO

NCG::ConfigGen

=cut

1;
