#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::LocalMetricSets::File;

use NCG::LocalMetricSets;
use vars qw(@ISA);

@ISA=("NCG::LocalMetricSets");

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);

    if (! defined $self->{DB_FILE} ) {
        $self->error("File name must be defined!");
        undef $self;
        return 0;
    }

    if (! -f $self->{DB_FILE})
    {
        $self->error("Can't find static file!");
        undef $self;
        return 0;
    }

    $self;
}

sub _loadMetricSets {
    my $self = shift;
    my $services = shift;
    my $line;
    my $fileHndl;
    
    if (!open ($fileHndl, $self->{DB_FILE})) {
        $self->error("Cannot open static file!");
        return 0;
    }
    while ($line = <$fileHndl>) {
        next if ($line =~ /^\s*#.*$/);
        if ($line =~ /^\s*(\S+?)!(\S+?)!(\S+)\s*$/) {
            my $action = $1;
            if ($action eq "SERVICE_METRICSET") {
                $services->{$2}->{$3} = 1;
            } else {
                $self->warning("Unknown command found: $1");
            }
        }
    }
    close ($fileHndl);
    
    1;
}

sub getData {
    my $self = shift;
    my $services = {};

    $self->_loadMetricSets ($services) || return 0;

    foreach my $host ($self->{SITEDB}->getHosts()) {
        foreach my $service ($self->{SITEDB}->getServices($host)) {
            if (exists $services->{$service})
            {
                foreach my $metricSet (keys %{$services->{$service}}) {
                    $self->{SITEDB}->addMetricSet($host, $metricSet, $service);
                }
            }
        }
    }

    return 1;
}

=head1 NAME

NCG::LocalMetricSets::File

=head1 DESCRIPTION

The NCG::LocalMetricSets::File module extends NCG::LocalMetricSets module.
Module extracts metric sets information from file with list of tuples:
  # this is a comment
  SERVICE_METRICSET!service!metricset
  SERVICE_METRICSET!service!metricset

=head1 SYNOPSIS

  use NCG::LocalMetricSets::File;

  my $lms = NCG::LocalMetricSets::File->new( { SITEDB=> $sitedb,
                                        DB_FILE => '/path/to/metricsetlist' } );

  $lms->getData();

=cut

=head1 METHODS

=over

=item C<new>

  $siteInfo = NCG::LocalMetricSets::File->new( $options );

Creates new NCG::LocalMetricSets::File instance. Argument $options is hash
reference that can contains following elements:
  DB_FILE - location of file with list of tuples (no default)

=back

=head1 SEE ALSO

NCG::LocalMetricSets

=cut

1;
