#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::LocalMetrics::File;

use NCG::LocalMetrics;
use vars qw(@ISA);

@ISA=("NCG::LocalMetrics");

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);

    if (! defined $self->{DB_FILE} ) {
        $self->error("File name must be defined!");
        undef $self;
        return 0;
    }

    if (! -f $self->{DB_FILE})
    {
        $self->error("Can't find static file!");
        undef $self;
        return 0;
    }
    
    $self;
}

sub _loadMetrics {
    my $self = shift;
    my $metricSets = shift;
    my $metrics = shift;
    my $line;
    my $fileHndl;
    
    if (!open ($fileHndl, $self->{DB_FILE})) {
        $self->error("Cannot open static file!");
        return 0;
    }

    while ($line = <$fileHndl>) {
        next if ($line =~ /^\s*#.*$/);
        if ($line =~ /^\s*(\S+?)!(\S+?)(!(\S+?))?(!(\S+))?\s*$/) {
            my $action = $1;
            if ($action eq "METRICSET_METRIC") {
                next if (!defined $4);
                $metricSets->{$2}->{$4} = 1 unless (exists $metricSets->{$2}->{$4});
            } elsif ($action eq "METRIC_PROBE") {
                next if (!defined $4);
                $metrics->{$2}->{probe} = $4;
            } elsif ($action eq "METRIC_DOCURL") {
                next if (!defined $4);
                $metrics->{$2}->{docurl} = $4;
            } elsif ($action eq "METRIC_NATIVE") {
                next if (!defined $4);
                $metrics->{$2}->{native} = $4;
            } elsif ($action eq "METRIC_VO") {
                $metrics->{$2}->{vo} = 1;
            } elsif ($action eq "METRIC_CONFIG") {
                next if (!defined $4 || !defined $6);
                $metrics->{$2}->{config}->{$4} = $6;
            } elsif ($action eq "METRIC_DEPENDENCY") {
                next if (!defined $4 || !defined $6);
                $metrics->{$2}->{dependencies}->{$4} = $6;
            } elsif ($action eq "METRIC_ATTRIBUTE") {
                next if (!defined $4 || !defined $6);
                $metrics->{$2}->{attributes}->{$4} = $6;
            } else {
                $self->warning("Unknown command found: $1");
            }
        }
    }
    close ($fileHndl);
    
    1;
}

sub getData {
    my $self = shift;
    my $metricSets = {};
    my $metrics = {};
    
    $self->_loadMetrics ($metricSets, $metrics) || return 0;

    foreach my $host ($self->{SITEDB}->getHosts()) {
        foreach my $service ($self->{SITEDB}->getMetricSets($host)) {
            if (exists $metricSets->{$service})
            {
                foreach my $metric (keys %{$metricSets->{$service}}) {
                    my $metricRef = $metrics->{$metric};
                    my $confRef = {};
                    my $depRef = {};
                    my $attrRef = {};
    
                    $confRef = {%{$metricRef->{config}}} if (exists $metricRef->{config});
                    $attrRef = {%{$metricRef->{attribute}}} if (exists $metricRef->{attribute});
                    $depRef = {%{$metricRef->{dependency}}} if (exists $metricRef->{dependency});

                    $self->{SITEDB}->addLocalMetric($host,
                                                    $metric,
                                                    $service,
                                                    $metricRef->{probe},
                                                    $metricRef->{native},
                                                    $metricRef->{vo},
                                                    $confRef,
                                                    $attrRef,
                                                    $depRef,
                                                    $metricRef->{docurl});
                }
            }
        }
    }
}

=head1 NAME

NCG::LocalMetricSets::File

=head1 DESCRIPTION

The NCG::LocalMetrics::File module extends NCG::LocalMetrics module.
Module extracts metric information from file with list of tuples:
  # this is a comment
  # association between metric and metricset
  METRICSET_METRIC!metricset!metric
  # probe used for the metric
  METRIC_PROBE!metric!probe
  # url with probe documentation
  METRIC_DOCURL!metric!url
  # metric is gahered by monitoring system's native probe
  METRIC_NATIVE!metric!native
  # metric is VO dependent
  METRIC_VO!metric
  # set config parameter for metric (e.g. timeout)
  METRIC_CONFIG!metric!config!value
  # set metricParent which which metric depend on
  METRIC_DEPENDENCY!metric!metricParent!value
  # define which attribute metric requires
  METRIC_ATTRIBUTE!metric!attribute!value

=head1 SYNOPSIS

  use NCG::LocalMetrics::File;

  my $lms = NCG::LocalMetrics::File->new( { SITEDB=> $sitedb,
                                    DB_FILE => '/path/to/metriclist' } );

  $lms->getData();

=cut

=head1 METHODS

=over

=item C<new>

  $siteInfo = NCG::LocalMetrics::File->new( $options );

Creates new NCG::LocalMetrics::File instance. Argument $options is hash
reference that can contains following elements:
  DB_FILE - location of file with list of tuples (no default)

=back

=head1 SEE ALSO

NCG::LocalMetrics

=cut

1;
