#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::LocalMetricsAttrs::Active;

use NCG::LocalMetricsAttrs;
use strict;
use Net::LDAP;
use vars qw(@ISA);

@ISA=("NCG::LocalMetricsAttrs");

my %MAP_SERVICETYPE_PORT = (
    BDII => 'BDII_PORT',
    ResourceBroker => 'RB_PORT',
    gsiftp => 'GRIDFTP_PORT',
    Tomcat => 'TOMCAT_PORT',
    'MDS-GIIS' => 'MDS_PORT',
    'globus-GRAM' => 'GRAM_PORT',
    'SRM1' => 'SRM1_PORT',
    'SRM2' => 'SRM2_PORT',
    DPNS => 'DPNS_PORT',
    'it.infn.GridICE' => 'GRIDICE_PORT',
    MyProxy => 'MYPROXY_PORT',
    'gLite-LFC' => 'LFC_PORT',
    'org.glite.wms.NetworkServer' => 'WMS_PORT',
    'gLite-RGMA' => 'RGMA_PORT',
    'gLite-FTS-WS' => 'FTS_PORT',
    gsissh => 'VOBOX_PORT'
);

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);

    $self;
}

sub getData {
    my $self = shift;
	my $sitename = shift || $self->{SITENAME};

    foreach my $hostname ($self->{SITEDB}->getHosts()) {
		$self->_setDefaultPorts($hostname);

		# analysis of specific services and metricsets are in
		# separate functions to make things more clear
		#
		# TODO:
		#   - issue with dependent analysis (e.g. CAdistribution
		#   depends on SRM because of gsiftp)

        $self->_analyzeGliteVersion($hostname, $sitename) if (defined $self->{GLITE_VERSION});

        # services
        $self->_analyzeSRM($hostname, $sitename);
		$self->_analyzeLFC($hostname, $sitename);
		$self->_analyzeMON($hostname, $sitename);
        $self->_analyzeCE($hostname, $sitename);
        $self->_analyzeSE($hostname, $sitename);
		
        # metricsets
		$self->_analyzeCAdistribution($hostname, $sitename);
		$self->_analyzeBDII($hostname, $sitename);
        $self->_analyzeMDS($hostname, $sitename);
        
        $self->_setStaticAttrs($hostname, $sitename);
		
		# Set serviceURI for all METRICSETS
		foreach my $service ($self->{SITEDB}->getMetricSets($hostname)) {
			if (! $self->{SITEDB}->hasServiceURI($hostname, $service) ) {
                my $portname = $MAP_SERVICETYPE_PORT{$service};
				if (exists $MAP_SERVICETYPE_PORT{$service} && $self->{SITEDB}->hasAttribute($hostname, $portname)) {
					$self->{SITEDB}->metricSetServiceURI($hostname, $service, $hostname . ':' . $self->{SITEDB}->hostAttribute($hostname, $portname));
				} else {
					$self->{SITEDB}->metricSetServiceURI($hostname, $service, $hostname);
				}
			}
		}
    }
}

sub _analyzeGliteVersion {
	my $self = shift;
	my $hostname = shift;

    if ($self->{GLITE_VERSION} =~ m/(\d+)\.(\d+)\.(\d+)/) {
        my $minorVer = $2;
        # remove NetworkServer
        if ($minorVer > 0) {
            if ($self->{SITEDB}->hasService($hostname, "WMS")) {
                $self->{SITEDB}->removeMetricSet($hostname, "org.glite.wms.NetworkServer");
            }
            if ($self->{SITEDB}->hasService($hostname, "RB")) {
                $self->{SITEDB}->removeMetricSet($hostname, "ResourceBroker");
            }
        }
    }
}

######################################################
###
###  Analyze services
###
######################################################

sub _analyzeLFC {
	my $self = shift;
	my $hostname = shift;
	
    # LFC runs either BDII or MDS-GIIS
	if ($self->{SITEDB}->hasService($hostname, "LFC")) {
        my $bdiiPort = $self->{SITEDB}->hostAttribute($hostname, "BDII_PORT") || 2170;
        my $mdsPort = $self->{SITEDB}->hostAttribute($hostname, "MDS_PORT") || 2135;
        
		if (!$self->checkTcp($self->{SITEDB}->hostAddress($hostname), $bdiiPort)){
            $self->{SITEDB}->removeMetricSet($hostname, "BDII");
		}
		if (!$self->checkTcp($self->{SITEDB}->hostAddress($hostname), $mdsPort)) {
			$self->{SITEDB}->removeMetricSet($hostname, "MDS-GIIS");
		}
	}	
}
	
sub _analyzeSRM {
	my $self = shift;
	my $hostname = shift;

	if ($self->{SITEDB}->hasService($hostname, "SRM")) {
		# First remove the services for which we don't have a serviceuri BDII
        if (! $self->{SITEDB}->hasServiceURI($hostname, "SRM1")) {
			$self->{SITEDB}->removeMetricSet($hostname, "SRM1");
		}

        if (! $self->{SITEDB}->hasServiceURI($hostname, "SRM2")) {
			$self->{SITEDB}->removeMetricSet($hostname, "SRM2");
		}

		unless ($self->checkTcp($self->{SITEDB}->hostAddress($hostname), $self->{SITEDB}->hostAttribute($hostname, "GRIDFTP_PORT"))) {
			$self->{SITEDB}->removeMetricSet($hostname, "gsiftp");
		}
		
		# Check if we found a good SA, if not, remove local checks
		if (! $self->{SITEDB}->hasAttribute($hostname, "SE_PATH")) {
			$self->{SITEDB}->removeMetricSet($hostname, "SRM1");
			$self->{SITEDB}->removeMetricSet($hostname, "SRM2");
            $self->{SITEDB}->removeMetricSet($hostname, "DPNS");
		} else {
            # check if this is a DPM box
            foreach my $path ($self->{SITEDB}->hostAttributeVOs($hostname, "SE_PATH")) {
                if ($path !~ /^\/dpm\//) {
                    $self->{SITEDB}->removeMetricSet($hostname, "DPNS");
                }
                last;
            }
        }

        my $bdiiPort = $self->{SITEDB}->hostAttribute($hostname, "BDII_PORT") || 2170;
        my $mdsPort = $self->{SITEDB}->hostAttribute($hostname, "MDS_PORT") || 2135;
        
		if (!$self->checkTcp($self->{SITEDB}->hostAddress($hostname), $bdiiPort)){
            $self->{SITEDB}->removeMetricSet($hostname, "BDII");
		}
		if (!$self->checkTcp($self->{SITEDB}->hostAddress($hostname), $mdsPort)) {
			$self->{SITEDB}->removeMetricSet($hostname, "MDS-GIIS");
		}
	}
}

sub _analyzeMON {
	my $self = shift;
	my $hostname = shift;
	my $sitename = shift;
	
	if ($self->{SITEDB}->hasService($hostname, "MON")) {
		if ($self->{SITEDB}->hasAttribute($hostname, "GRIDICE_PORT")) {
            $self->{SITEDB}->hostAttribute($hostname, "GRIDICE_DN", "Mds-Vo-Name=local,O=Grid");
		} elsif ($self->checkTcp($self->{SITEDB}->hostAddress($hostname),2136)) {
            $self->{SITEDB}->hostAttribute($hostname, "GRIDICE_PORT", 2136);
            $self->{SITEDB}->hostAttribute($hostname, "GRIDICE_DN", "Mds-Vo-Name=local,O=Grid");
		} else {
            $self->{SITEDB}->removeMetricSet($hostname, "it.infn.GridICE");
		}
	}
}

sub _analyzeCE {
	my $self = shift;
	my $hostname = shift;
	my $sitename = shift;
	
    # GridFTP is closed on CE
    if ($self->{SITEDB}->hasService($hostname, "CE") || $self->{SITEDB}->hasService($hostname, "gLite-CE")) {
        if (! $self->checkTcp($self->{SITEDB}->hostAddress($hostname),$self->{SITEDB}->hostAttribute($hostname, "GRIDFTP_PORT"))) {
            $self->{SITEDB}->removeMetricSet($hostname, "gsiftp");
        }
    }	
}

sub _analyzeSE {
    my $self = shift;
	my $hostname = shift;
	my $sitename = shift;

    if ($self->{SITEDB}->hasService($hostname, "SE") && $self->{SITEDB}->hasAttribute($hostname, "SE_PATH")) {
        foreach my $path ($self->{SITEDB}->hostAttributeVOs($hostname, "SE_PATH")) {
            if ($path =~ /^\/pnfs\//) {
                $self->{SITEDB}->removeMetricSet($hostname, "CAdistribution");
                #$self->{HOSTS}->{$hostname}->{SE_TYPE} = "DCACHE";
            }
            #elsif ($path =~ /^\/dpm\//) {
            #    $self->{HOSTS}->{$hostname}->{SE_TYPE} = "DPM";
            #} elsif ($path =~ /^\/castor\//) {
            #    $self->{HOSTS}->{$hostname}->{SE_TYPE} = "CASTOR";
            #} else {
            #    $self->{HOSTS}->{$hostname}->{SE_TYPE} = "CLASSIC";
            #}
            last;
        }
    }
}

######################################################
###
###  Analyze metricsets
###
######################################################

sub _analyzeCAdistribution {
	my $self = shift;
	my $hostname = shift;
	my $sitename = shift;	
	
	if ($self->{SITEDB}->hasMetricSet($hostname, "CAdistribution")) {
		if ($self->{SITEDB}->hasMetricSet($hostname, "gsiftp")) {
            $self->{SITEDB}->metricSetServiceURI($hostname, "CAdistribution", $hostname.":".$self->{SITEDB}->hostAttribute($hostname, "GRIDFTP_PORT"));
		} else {
            $self->{SITEDB}->removeMetricSet($hostname, "CAdistribution");
		}
	}
}

sub _analyzeBDII {
	my $self = shift;
	my $hostname = shift;
	my $sitename = shift;
	
	# Set LDAP check parameter for BDII
	if ($self->{SITEDB}->hasMetricSet($hostname, "BDII")) {
		if ($self->{SITEDB}->hasService($hostname, "BDII")) {
            $self->{SITEDB}->hostAttribute($hostname, "BDII_DN", "Mds-Vo-Name=local,O=Grid");
		} elsif ($self->{SITEDB}->hasService($hostname, "Site-BDII")) {
			$self->{SITEDB}->hostAttribute($hostname, "BDII_DN", "GlueSiteUniqueID=$sitename,Mds-Vo-Name=$sitename,O=Grid");
		} elsif ($self->{SITEDB}->hasService($hostname, "DPM")) {
			$self->{SITEDB}->hostAttribute($hostname, "BDII_DN", "GlueSEUniqueID=$hostname,Mds-Vo-Name=resource,O=Grid");
		} elsif ($self->{SITEDB}->hasService($hostname, "LFC")) {
            $self->{SITEDB}->hostAttribute($hostname, "BDII_DN", "GlueServiceUniqueID=$hostname,Mds-Vo-Name=resource,O=Grid");
		} elsif ($self->{SITEDB}->hasService($hostname, "SRM")) {
			if ($self->{SITEDB}->hasMetricSet($hostname, "SRM2")) {
                $self->{SITEDB}->hostAttribute($hostname, "BDII_DN", "GlueServiceUniqueID=httpg://$hostname:".$self->{SITEDB}->hostAttribute($hostname,"SRM2_PORT")."/srm/managerv2,Mds-Vo-Name=resource,O=Grid");
			} else {
                $self->{SITEDB}->hostAttribute($hostname, "BDII_DN", "GlueServiceUniqueID=httpg://$hostname:".$self->{SITEDB}->hostAttribute($hostname,"SRM1_PORT")."/srm/managerv1,Mds-Vo-Name=resource,O=Grid");
			}
		} else {
#		    $node->{BDII_DN} = "GlueServiceUniqueID=glite-data-transfer-fts_".$hostname."_org.glite.ChannelManagement,Mds-Vo-Name=$sitename,O=Grid";
			$self->{SITEDB}->hostAttribute($hostname, "BDII_DN", "Mds-Vo-Name=resource,O=Grid");
		}
	}
}

sub _analyzeMDS {
	my $self = shift;
	my $hostname = shift;
	my $sitename = shift;	

	# Check if GRIS is opened & set LDAP check parameter for specific node type
	if ($self->{SITEDB}->hasMetricSet($hostname, "MDS-GIIS")) {
		if ( $self->checkTcp( $self->{SITEDB}->hostAddress($hostname), $self->{SITEDB}->hostAttribute($hostname, "MDS_PORT")) ) {
			if ($self->{SITEDB}->hasService($hostname, "CE") || $self->{SITEDB}->hasService($hostname, "gLite-CE")) {
				$self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "GlueClusterUniqueID=$hostname,Mds-Vo-Name=local,O=Grid");
			} elsif ($self->{SITEDB}->hasService($hostname, "SE")) {
                $self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "GlueSEUniqueID=$hostname,Mds-Vo-Name=local,O=Grid");
			} elsif ($self->{SITEDB}->hasService($hostname, "RB")) {
				$self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "GlueServiceUniqueID=$hostname:".$self->{SITEDB}->hostAttribute($hostname,"RB_PORT").",Mds-Vo-Name=local,O=Grid");
			} elsif ($self->{SITEDB}->hasService($hostname, "LFC")) {
				$self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "Mds-Vo-Name=local,O=Grid");
			} elsif ($self->{SITEDB}->hasService($hostname, "SRM")) {
				if ($self->{SITEDB}->hasMetricSet($hostname, "SRM2")) {
					$self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "GlueServiceUniqueID=httpg://$hostname:".$self->{SITEDB}->hostAttribute($hostname,"SRM2_PORT")."/srm/managerv2,Mds-Vo-Name=local,O=Grid");
				} else {
					$self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "GlueServiceUniqueID=httpg://$hostname:".$self->{SITEDB}->hostAttribute($hostname,"SRM1_PORT")."/srm/managerv1,Mds-Vo-Name=local,O=Grid");
				}
			} elsif ($self->{SITEDB}->hasService($hostname, "PROX")) {
				$self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "GlueServiceUniqueID=$hostname:".$self->{SITEDB}->hostAttribute($hostname,"MYPROXY_PORT").",mds-vo-name=local,o=grid");
			} elsif ($self->{SITEDB}->hasMetricSet($hostname, "it.infn.GridICE")) {
				$self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "GlueServiceUniqueID=$hostname:".$self->{SITEDB}->hostAttribute($hostname,"GRIDICE_PORT").",mds-vo-name=local,o=grid");
            } elsif ($self->{SITEDB}->hasService($hostname, "VOBOX")) {
                $self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "GlueServiceUniqueID=gsissh://$hostname:".$self->{SITEDB}->hostAttribute($hostname,"VOBOX_PORT").",mds-vo-name=local,o=grid");
            } else {
				$self->{SITEDB}->hostAttribute($hostname, "MDS_DN", "mds-vo-name=local,o=grid");
			}
		} else {
			$self->{SITEDB}->removeMetricSet($hostname, "MDS-GIIS");
		}
	}
}

sub _setDefaultPorts {
	my $self = shift;
	my $hostname = shift;

	# Set GridFTP port needed for native checks
	if ( $self->{SITEDB}->hasMetricSet($hostname, "BDII") && ! $self->{SITEDB}->hasAttribute($hostname, "BDII_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "BDII_PORT", 2170);
	}
    
	# Set GridFTP port needed for native checks
	if ( $self->{SITEDB}->hasMetricSet($hostname, "MDS-GIIS") && ! $self->{SITEDB}->hasAttribute($hostname, "MDS_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "MDS_PORT", 2135);
	}    
	
	# Set GridFTP port needed for native checks
	if ( $self->{SITEDB}->hasMetricSet($hostname, "gsiftp") && ! $self->{SITEDB}->hasAttribute($hostname, "GRIDFTP_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "GRIDFTP_PORT", 2811);
	}

	# Set GRAM port if not extracted from BDII
	if ( $self->{SITEDB}->hasMetricSet($hostname, "globus-GRAM") && ! $self->{SITEDB}->hasAttribute($hostname, "GRAM_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "GRAM_PORT", 2119);
	}
    
    # Set RB port if not extracted from BDII
	if ( $self->{SITEDB}->hasMetricSet($hostname, "ResourceBroker") && ! $self->{SITEDB}->hasAttribute($hostname, "RB_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "RB_PORT", 7772);
	}

    # Set WMS port if not extracted from BDII
	if ( $self->{SITEDB}->hasMetricSet($hostname, "org.glite.wms.NetworkServer") && ! $self->{SITEDB}->hasAttribute($hostname, "WMS_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "WMS_PORT", 7772);
	}

    # Set MyProxy port if not extracted from BDII
    if ( $self->{SITEDB}->hasMetricSet($hostname, "MyProxy") && ! $self->{SITEDB}->hasAttribute($hostname, "MYPROXY_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "MYPROXY_PORT", 7512);
	}
	
	# Set RGMA port if not extracted from BDII
    if ( $self->{SITEDB}->hasMetricSet($hostname, "gLite-RGMA") && ! $self->{SITEDB}->hasAttribute($hostname, "RGMA_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "RGMA_PORT", 8443);
	}

	if ( $self->{SITEDB}->hasMetricSet($hostname, "Tomcat") && ! $self->{SITEDB}->hasAttribute($hostname, "TOMCAT_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "TOMCAT_PORT", 8443);
	}

	# Set LL port if not extracted from somewhere?
    if ( $self->{SITEDB}->hasMetricSet($hostname, "org.glite.LocalLogger") && ! $self->{SITEDB}->hasAttribute($hostname, "LL_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "LL_PORT", 9002);
	}
	
	# Set WMProxy port if not extracted from somewhere?
	if ( $self->{SITEDB}->hasMetricSet($hostname, "org.glite.wms.WMProxy") && ! $self->{SITEDB}->hasAttribute($hostname, "WMPROXY_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "WMPROXY_PORT", 7443);
	}
	
	# Set SRM1 data if not extracted from BDII
	if ( $self->{SITEDB}->hasMetricSet($hostname, "SRM1") && ! $self->{SITEDB}->hasAttribute($hostname, "SRM1_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "SRM1_PORT", 8443);
	}

	# Set SRM2 data if not extracted from BDII
    if ( $self->{SITEDB}->hasMetricSet($hostname, "SRM2") && ! $self->{SITEDB}->hasAttribute($hostname, "SRM2_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "SRM2_PORT", 8446);
	}
	
	# Set VOBOX data if not extracted from BDII
    if ( $self->{SITEDB}->hasMetricSet($hostname, "gsissh") && ! $self->{SITEDB}->hasAttribute($hostname, "VOBOX_PORT") ) {
		$self->{SITEDB}->hostAttribute($hostname, "VOBOX_PORT", 1975);
	}
}

# TODO: 
#  move these to localdb file
sub _setStaticAttrs {
	my $self = shift;
	my $hostname = shift;
	my $VDT_LOCATION = "test";
    
    $self->{SITEDB}->hostAttribute($hostname, "VDT_LOCATION", $VDT_LOCATION);
    $self->{SITEDB}->hostAttribute($hostname, "OSG_TEST_FILE", "=/etc/group ");
}

=head1 NAME

NCG::SiteInfo::Active

=head1 DESCRIPTION

The NCG::LocalMetricsAttrs::Active module extends NCG::LocalMetricsAttrs
module. Module extracts detailed metric information by using various
heuristics. Heuristics for each service and metricset are placed in
separate method _analyzeXXX.

=head1 SYNOPSIS

  use NCG::LocalMetricsAttrs::LDAP;

  my $metricInfo = NCG::LocalMetricsAttrs::Active->new();

  $metricInfo->getData();

=cut

=head1 METHODS

=over

=item C<new>

  my $siteInfo = NCG::SiteInfo::Active->new($options); 

Creates new NCG::SiteInfo::Active instance. 

=back

=head1 SEE ALSO

NCG::LocalMetricsAttrs

=cut

1;
