#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::LocalMetricsAttrs::File;

use NCG::LocalMetricsAttrs;
use strict;
use vars qw(@ISA);

@ISA=("NCG::LocalMetricsAttrs");

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);
    
    if (! defined $self->{DB_FILE} ) {
        $self->error("File name must be defined!");
        undef $self;
        return 0;
    }

    if (! -f $self->{DB_FILE})
    {
        $self->error("Can't find static file!");
        undef $self;
        return 0;
    }
    
    $self;
}

sub _loadMetricAttrs {
    my $self = shift;
    my $attributes = shift;
    my $hosts = shift;
    my $services = shift;
    my $line;
    my $fileHndl;
    
    if (!open ($fileHndl, $self->{DB_FILE})) {
        $self->error("Cannot open static file!");
        return 0;
    }
    while ($line = <$fileHndl>) {
        next if ($line =~ /^\s*#.*$/);
        if ($line =~ /^\s*(\S+?)!(\S+?)!(\S+?)(!(\S+))?\s*$/) {
            my $action = $1;
            if ($action eq "ATTRIBUTE") {
                $attributes->{$2} = $3;
            } elsif ($action eq "HOST_ATTRIBUTE") {
                $hosts->{$2}->{$3} = $5;
            } elsif ($action eq "SERVICE_ATTRIBUTE") {
                $services->{$2}->{$3} = $5;
            } else {
                $self->warning("Unknown command found: $1");
            }
        }
    }
    close ($fileHndl);
    
    1;
}

sub getData {
    my $self = shift;
    my $attributes = {};
    my $hosts = {};
    my $services = {};
    my $attribute;
    
    $self->_loadMetricAttrs ($attributes, $hosts, $services) || return 0;

    foreach my $hostname ($self->{SITEDB}->getHosts()) {
        # add default attributes
        foreach $attribute (keys %$attributes)
        {
            $self->{SITEDB}->hostAttribute($hostname, $attribute, $attributes->{$attribute});
        }
        
        # add host specific attributes
        if (exists $hosts->{$hostname})
        {
            foreach $attribute (keys %{$hosts->{$hostname}})
            {
                $self->{SITEDB}->hostAttribute($hostname, $attribute, $hosts->{$hostname}->{$attribute});
            }
        }
        
        # add service specific attributes
        foreach my $service ($self->{SITEDB}->getServices($hostname)) {
            if (exists $services->{$service})
            {
                foreach $attribute (keys %{$services->{$service}}) {
                    $self->{SITEDB}->hostAttribute($hostname, $attribute, $services->{$service}->{$attribute});
                }
            }
        }
    }
    
    1;
}


=head1 NAME

NCG::LocalMetricsAttrs::File

=head1 DESCRIPTION

The NCG::LocalMetricsAttrs::File module extends NCG::LocalMetricsAttrs
module. Module extracts detailed metric information from file with
list of tuples:
  # this is a comment
  # add attribute value to all hosts
  ATTRIBUTE!name!value
  # add attribute value to defined host
  HOST_ATTRIBUTE!host!name!value
  # add attribute to each host which contains defined service
  SERVICE_ATTRIBUTE!service!name!value

information services (e.g. BDII, Globus MDS).

=head1 SYNOPSIS

  use NCG::LocalMetricsAttrs::LDAP;

  my $lma = NCG::LocalMetricSets::File->new( { SITEDB=> $sitedb,
                                        DB_FILE => '/path/to/metricsetlist' } );

  $lma->getData();

=cut

=head1 METHODS

=over

=item C<new>

  my $lma = NCG::LocalMetricsAttrs::File->new( $options );

Creates new NCG::LocalMetricsAttrs::File instance. Argument $options is hash
reference that can contains following elements:
  DB_FILE - location of file with list of tuples (no default)

=back

=head1 SEE ALSO

NCG::LocalMetricsAttrs

=cut

1;
