#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::LocalMetricsAttrs::LDAP;

use NCG::LocalMetricsAttrs;
use strict;
use Net::LDAP;
use vars qw(@ISA);

@ISA=("NCG::LocalMetricsAttrs");

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);
    
    # set default values
    if (! $self->{LDAP_ADDRESS}) {
        if ( ! ($self->{LDAP_ADDRESS} = $self->{SITEDB}->siteLDAP)) {
            $self->error("LDAP_ADDRESS must be defined!");
            undef $self;
            return;
        }
    }
    # default LDAP port is BDII
    if (! defined $self->{LDAP_PORT} ) {
        $self->{LDAP_PORT} = 2170;
    }
    
    $self;
}

sub getData {
    my $self = shift;
    my $sitename = shift || $self->{SITENAME} || "";

    #if ($sitename) {
    #    $self->error("Sitename must be defined!");
    #    return 0;
    #}

    my $basedn;
    my $mesg;
    my $hostname;
    my $port;
    my $entry;
    my $voEntry;
    
    if (! $self->{LDAP_BASEDN} ) {
        if ($self->{LDAP_PORT} == 2135) {
            $self->{LDAP_BASEDN} = "Mds-Vo-Name=local, O=Grid";
        } elsif ($self->{LDAP_PORT} == 2170) {
            $self->{LDAP_BASEDN} = "Mds-Vo-Name=$sitename, O=Grid";
        } else {
            $self->warning("Base DN not defined, port ". $self->{LDAP_PORT} ." is not BDII or MDS. Assuming 'O=Grid'");
            $self->{LDAP_BASEDN} = "O=Grid";
        }
    }

    #verbose("Getting info from BDII $bdii for site $sitename:\n");
    
    my $ldap = Net::LDAP->new( $self->{LDAP_ADDRESS}, port => $self->{LDAP_PORT}, timeout => $self->{TIMEOUT}, async => 1 );
    if (! defined $ldap ) {
        $self->error("Cannot connect to $self->{LDAP_ADDRESS}:$self->{LDAP_PORT}");
        return 0;
    }    
    $mesg = $ldap->bind( anonymous => 1 );
    if ($mesg->code) {
        $self->error("Could not bind to LDAP: ".$mesg->error);
        return 0;
    }

    # Get SE information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(objectClass=GlueSE)",
                            attrs=> ['GlueSEUniqueID'],
                            timelimit => 60 );
    
    if ($mesg->code) {
        $self->error("Could not fetch SE info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            $hostname = $entry->get_value('GlueSEUniqueID');
            my $path;

            # Get remote path that selected VO can use
            my $mesg1 = $ldap->search ( base => $self->{LDAP_BASEDN},
                                        filter => "(&(objectClass=GlueSA)(GlueChunkKey=GlueSEUniqueID=$hostname))",
                                        attrs => ['GlueSAAccessControlBaseRule','GlueSAPath'],
                                        timelimit => 60 );
            if ($mesg1->code) {
                $self->error("Could not fetch SE VO info from LDAP: ".$mesg->error);    
            } else {
                foreach my $voEntry ($mesg1->entries()) {
                    my $voname = lc($voEntry->get_value('GlueSAAccessControlBaseRule'));
                    $self->{SITEDB}->hostAttributeVO($hostname, "SE_PATH", $voname, $voEntry->get_value('GlueSAPath'));
                }
            }

            # GridFTP specific parameter
            $mesg1 = $ldap->search ( base => $self->{LDAP_BASEDN},
                                    filter => "(&(&(objectClass=GlueSEAccessProtocol)(GlueChunkKey=GlueSEUniqueID=$hostname))(GlueSEAccessProtocolType=gsiftp))",
                                    attrs => ['GlueSEAccessProtocolPort'],
                                    timelimit => 60 );
		
            next if ($mesg1->code);
            if ($mesg1->count != 0) {
                my $port = $mesg1->entry(0)->get_value('GlueSEAccessProtocolPort');
                $self->{SITEDB}->hostAttribute($hostname, "GRIDFTP_PORT", $port);
                $self->{SITEDB}->metricSetServiceURI($hostname, "gsiftp", "$hostname:$port");
            } else {
                $self->{SITEDB}->removeMetricSet($hostname, "gsiftp");
            }                    
        }
    }

    # Get CE information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(objectClass=GlueCE)",
                            attrs=> ['GlueCEUniqueID'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch CE info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueCEUniqueID') =~ /([-_.A-Za-z0-9]+):(\d+)\/(jobmanager|blahp)-([-_.A-Za-z0-9]+)-([-_.A-Za-z0-9]+)/) {
                $hostname = $1;
                $port = $2;
                $self->{SITEDB}->hostAttribute($hostname, "GRAM_PORT", $port);
                $self->{SITEDB}->metricSetServiceURI($hostname, "globus-GRAM", "$hostname:$port");
            }
        }
    }

    # check CE version
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(objectClass=GlueSubCluster)",
                            attrs=> ['GlueSubClusterUniqueID', 'GlueHostApplicationSoftwareRunTimeEnvironment'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch CE info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            $hostname = $entry->get_value('GlueSubClusterUniqueID');
            foreach my $attr ($entry->get_value('GlueHostApplicationSoftwareRunTimeEnvironment')) {
                # CE is version 3.1, modifications are needed
                if ($attr =~ /GLITE-3_1(.*)/) {
                    $self->{SITEDB}->removeMetricSet($hostname, "MDS-GIIS");
                    $self->{SITEDB}->addMetricSet($hostname, "BDII", "CE");
                }
            }
        }
    }

    # Get SRM information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(|(GlueServiceType=srm_v1)(GlueServiceType=srm)))",
                            attrs=> ['GlueServiceURI','GlueServiceVersion'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch SRM info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceURI') =~ /httpg:\/\/([-_.A-Za-z0-9]+):(\d+)\/srm\/managerv\d+/) {
                $hostname = $1;
                $port = $2;

                if($entry->get_value('GlueServiceVersion') eq '2.2.0') {
                    # SRMv2
                    $self->{SITEDB}->hostAttribute($hostname, "SRM2_PORT", $port);
                    $self->{SITEDB}->metricSetServiceURI($hostname, "SRM2", "httpg://$hostname:$port/srm/managerv2");
                } else { # Allow anything else to be SRMv1 to deal with bad information providers
                    $self->{SITEDB}->hostAttribute($hostname, "SRM1_PORT", $port);
                    $self->{SITEDB}->metricSetServiceURI($hostname, "SRM1", "httpg://$hostname:$port/srm/managerv1");
                }

            }
        }
    }

    # Get Resource Broker information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=ResourceBroker))",
                            attrs=> ['GlueServiceEndpoint'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch RB info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceEndpoint') =~ /([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $port = $2;
                $self->{SITEDB}->hostAttribute($hostname, "RB_PORT", $port);
                $self->{SITEDB}->metricSetServiceURI($hostname, "ResourceBroker", "$hostname:$port");
            }
        }
    }

    # Get Resource Broker information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=org.glite.wms))",
                            attrs=> ['GlueServiceUniqueID'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch WMS info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceUniqueID') =~ /([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $port = $2;
                $self->{SITEDB}->hostAttribute($hostname, "WMS_PORT", $port);
                $self->{SITEDB}->metricSetServiceURI($hostname, "org.glite.wms.NetworkServer", "$hostname:$port");
            }
        }
    }
    
    # Get VOBOX information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=VOBOX))",
                            attrs=> ['GlueServiceEndpoint'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch VOBOX info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceUniqueID') =~ /gsissh:\/\/([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $port = $2;
                $self->{SITEDB}->hostAttribute($hostname, "VOBOX_PORT", $port);
                $self->{SITEDB}->metricSetServiceURI($hostname, "gsissh", "$hostname:$port");
            }
        }
    }

    # Get MyProxy information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=MyProxy))",
                            attrs=> ['GlueServiceEndpoint'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch PROX info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceEndpoint') =~ /([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $port = $2;
                $self->{SITEDB}->hostAttribute($hostname, "MYPROXY_PORT", $port);
                $self->{SITEDB}->metricSetServiceURI($hostname, "MyProxy", "$hostname:$port");
            }
        }
    }

    # Get MON boxes information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=gridice))",
                            attrs=> ['GlueServiceUniqueID'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch MON info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceUniqueID') =~ /([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $port = $2;
                $self->{SITEDB}->hostAttribute($hostname, "GRIDICE_PORT", $port);
                $self->{SITEDB}->metricSetServiceURI($hostname, "it.infn.GridICE", "$hostname:$port");
            }
        }
    }
    
    # Get top & site BDII information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(|(GlueServiceType=bdii_top)(GlueServiceType=bdii_site)))",
                            attrs=> ['GlueServiceUniqueID'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch TOP BDII info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceUniqueID') =~ /([-_.A-Za-z0-9]+):(\d+)\/bdii\-(top|site)/) {
                $hostname = $1;
                $port = $2;
                $self->{SITEDB}->hostAttribute($hostname, "BDII_PORT", $port);
                $self->{SITEDB}->metricSetServiceURI($hostname, "BDII", "$hostname:$port");
            }
        }
    }
    
    1;
}



sub baseDN {
    my $self = shift;
    my $value = shift;
    $self->{LDAP_BASEDN} = $value if (defined $value);
    $self->{LDAP_BASEDN};
}

sub address {
    my $self = shift;
    my $value = shift;
    $self->{LDAP_ADDRESS} = $value if (defined $value);
    $self->{LDAP_ADDRESS};
}

sub port {
    my $self = shift;
    my $value = shift;
    $self->{LDAP_PORT} = $value if (defined $value);
    $self->{LDAP_PORT};
}

=head1 NAME

NCG::LocalMetricsAttrs::LDAP

=head1 DESCRIPTION

The NCG::LocalMetricsAttrs::LDAP module extends NCG::LocalMetricsAttrs
module. Module extracts detailed metric information from LDAP-based 
information services (e.g. BDII, Globus MDS).

=head1 SYNOPSIS

  use NCG::LocalMetricsAttrs::LDAP;

  my $metricInfo = NCG::LocalMetricsAttrs::LDAP->new({LDAP_ADDRESS=>'ldap.server.com',
                                           LDAP_PORT=>2170,
                                           LDAP_BASEDN=>'O=Grid', SITEDB=>$siteDb});

  $siteInfo->getData();

=cut

=head1 METHODS

=over

=item C<new>

  my $siteInfo = NCG::LocalMetricsAttrs::LDAP->new($options);

Creates new NCG::LocalMetricsAttrs::LDAP instance. Argument $options is 
hash reference that can contains following elements:
  LDAP_ADDRESS - address of LDAP server (default: localhost)
  LDAP_PORT - port of LDAP server (default: 2170)
  LDAP_BASEDN - basedn used when querying LDAP server

=item C<address>

  print $siteInfo->address;
  $siteInfo->address('ldap.server.com');

Accessor method for property address.

=item C<port>

  print $siteInfo->port;
  $siteInfo->port(2135);

Accessor method for property port.

=item C<baseDN>

  print $siteInfo->baseDN;
  $siteInfo->baseDN('Mds-Vo-Name=myegeesite,O=Grid');

Accessor method for property baseDN.

=back

=head1 SEE ALSO

NCG::LocalMetricsAttrs

=cut

1;
