#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::LocalRules::File;

use NCG::LocalRules;
use strict;
use vars qw(@ISA);

@ISA=("NCG::LocalRules");

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);

    if (! defined $self->{DB_FILE} ) {
        $self->error("File name must be defined!");
        undef $self;
        return 0;
    }

    if (! -f $self->{DB_FILE})
    {
        $self->error("Can't find static file!");
        undef $self;
        return 0;
    }
    
    $self;
}

sub getData
{
    my $self = shift;
    my $line;
    my $fileHndl;
    
    if (!open ($fileHndl, $self->{DB_FILE})) {
        $self->error("Cannot open static file!");
        return 0;
    }
    while ($line = <$fileHndl>) {
        next if ($line =~ /^#.*$/);
        if ($line =~ /^\s*(\S+?)!(\S+?)(!(\S+))?\s*$/) {
            my $action = $1;
            if ($action eq "REMOVE_HOST") {
                $self->{SITEDB}->removeHost($2);
            } elsif ($action eq "REMOVE_SERVICE") {
                $self->{SITEDB}->removeMetricSet($2, $4, 1);
            } elsif ($action eq "ADD_LB") {
                $self->{SITEDB}->addLBNode($2, $4);
            } elsif ($action eq "REMOVE_LB") {
                $self->{SITEDB}->removeLBNode($2, $4);
            } elsif ($action eq "ADD_CONTACT") {
                $self->{SITEDB}->addContact($2);
            } elsif ($action eq "ADD_HOSTCONTACT") {
                $self->{SITEDB}->addHostContact($2,$4);
            } else {
                $self->warning("Unknown command found: $1");
            }
        }
    }
    close ($fileHndl);
}

=head1 NAME

NCG::LocalRules::File

=head1 DESCRIPTION

Module NCG::LocalRules::File applies local rules to gathered
information about site. Rules are stored in a simple text file.

Available local rules are:
- remove host
- remove service from the host
- add/remove load balancing nodes

Format:
  # remove the host
  REMOVE_HOST!host
  # remove the host/service
  REMOVE_SERVICE!host!service
  # add load balancing node
  ADD_LB!host!node
  # remove load balancing node
  REMOVE_LB!host!node
  # add contact to all hosts
  ADD_CONTACT!email@email.com
  # add contact to all hosts
  ADD_HOSTCONTACT!host!email@email.com
  
=head1 SYNOPSIS

  use NCG::LocalRules::File;

  my $siteInfo = NCG::LocalRules::File->new( { DB_FILE => '/path/to/rules' } );
  
  $siteInfo->getData();

=cut

1;