#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::RemoteMetrics::NPM;

use strict;
use NCG::RemoteMetrics;
use LWP::UserAgent;
use XML::DOM;

use vars qw(@ISA);

@ISA=("NCG::RemoteMetrics");

my $DEFAULT_NPM_ROOT_URL = "https://ccenoc.in2p3.fr/DownCollector/";

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);
    
    # set default values
    if (! defined $self->{NPM_ROOT_URL}) {
        $self->{NPM_ROOT_URL} = $DEFAULT_NPM_ROOT_URL;
    }
    
    $self;
}

sub NPMRootUrl {
    my $self = shift;
    my $value = shift;
    $self->{NPM_ROOT_URL} = $value if (defined $value);
    $self->{NPM_ROOT_URL};
}

sub defaultNPMRootUrl {
    my $self = shift;
    return $DEFAULT_NPM_ROOT_URL;
}

sub getData {
    my $self = shift;
    my $sitename = shift || $self->{SITENAME};;

    my $ua = LWP::UserAgent->new;
    $ua->agent("NCG::RemoteMetrics::NPM");

    my $req = HTTP::Request->new(POST => $self->{NPM_ROOT_URL}.'EMS/EMSMonitoringService.php?proc=getAllMetrics&sitename='.$sitename);
    my $res = $ua->request($req);

    if (!$res->is_success) {
        $self->error("Could not get metrics from NPM: ".$res->status_line);
        return 0;
    }

    my $parser = new XML::DOM::Parser(ErrorContext => 2);
    my $doc = $parser->parse($res->content);

    foreach my $host ($doc->getElementsByTagName("Host")) {
        my $hostname = $host->getAttribute("name");
        
        foreach my $metric ($host->getElementsByTagName("Service")) {
            my $type = $metric->getAttribute("type");
            
            $self->{SITEDB}->addRemoteMetric($hostname, $self->_getMetricName($type), "NPM", $self->_getMetricUrl($hostname,$type));
        }        
    }

    $doc->dispose;

    1;
}

sub _getMetricName {
    my $self = shift;
    my $ext = shift;
    
    return "org.egee.npm.$ext";
}

sub _getMetricUrl {
    my $self = shift;
    my $hostname = shift;
    my $type = shift;
    
    return $self->{NPM_ROOT_URL}."?nd=$type,$hostname";
}

=head1 NAME

NCG::RemoteMetrics::NPM

=head1 DESCRIPTION

The NCG::RemoteMetrics::NPM module extends NCG::RemoteMetrics module.
Module extracts list of remote metric from NPM (DownCollector).

=head1 SYNOPSIS

  use NCG::RemoteMetrics::NPM;

  my $ncg = NCG::RemoteMetrics::NPM->new( { SITEDB=> $sitedb } );

  $ncg->getData();

=cut

=head1 METHODS

=over

=item C<new>

  $ncg = NCG::RemoteMetrics::NPM->new( $options );

Creates new NCG::RemoteMetrics::NPM instance. Argument $options is hash
reference that can contain following elements:
  NPM_ROOT_URL - root URL used for Down Collector interface
  (default: https://ccenoc.in2p3.fr/DownCollector/)
  
=item C<NPMRootUrl>

  print $ncg->NPMRootUrl;
  $ncg->NPMRootUrl('https://ccenoc.in2p3.fr/DownCollector2/');
  
Accessor method for property NPM_ROOT_URL.  

=item C<defaultNPMRootUrl>

  print $ncg->NPMRootUrl;

Read only accessor method for default value of property
NPM_ROOT_URL.

=back

=head1 SEE ALSO

NCG::RemoteMetrics

=cut


1;
