#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::RemoteMetrics::SAM;

use strict;
use NCG::RemoteMetrics;
use LWP::UserAgent;
use XML::DOM;

use vars qw(@ISA);

@ISA=("NCG::RemoteMetrics");

my $DEFAULT_SAM_ROOT_URL = "http://lcg-sam.cern.ch:8080/same-pi/";

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);
    
    # set default values
    if (! $self->{SAM_ROOT_URL}) {
        $self->{SAM_ROOT_URL} = $DEFAULT_SAM_ROOT_URL;
    }
    
    $self;
}

sub SAMRootUrl {
    my $self = shift;
    my $value = shift;
    $self->{SAM_ROOT_URL} = $value if (defined $value);
    $self->{SAM_ROOT_URL};
}

sub SAMVOs {
    my $self = shift;
    my $value = shift;
    $self->{SAM_VOS} = $value if (defined $value);
    $self->{SAM_VOS};
}

sub SAMIncludeAll {
    my $self = shift;
    my $value = shift;
    $self->{SAM_INCLUDE_ALL} = $value if (defined $value);
    $self->{SAM_INCLUDE_ALL};
}

sub defaultSAMRootUrl {
    my $self = shift;
    return $DEFAULT_SAM_ROOT_URL;
}

sub getData {
    my $self = shift;
    my $sitename = shift || $self->{SITENAME};
    
    my @supportedVos;
    
    if ($self->{SAM_VOS}) {
        @supportedVos = split (/\s*,\s*/, $self->{SAM_VOS});
        unless (@supportedVos) {
            $self->error("Parameter SAM_VOS is not properly formatted: " . $self->{SAM_VOS});
            return;
        }
    } else {
        @supportedVos = $self->_getSupportedVOs($sitename);
        unless (@supportedVos) {
            return;
        }
    }
    
    my $criticalMetrics;
    my $metricDocs;
    if ($self->{SAM_INCLUDE_ALL}) {
        # documentation URL of critical checks is given with them so there is no need to
        # run this query each time
        $metricDocs = $self->_getMetricDocUrls;
    } else {
        ($criticalMetrics, $metricDocs) = $self->_getCriticalMetrics(\@supportedVos);

        if (!$criticalMetrics || !$metricDocs) {
            return;
        }
    }

    my $ua = LWP::UserAgent->new;
    $ua->agent("NCG::RemoteMetrics::SAM");

    foreach my $vo (@supportedVos) {
        my $req = HTTP::Request->new(POST => $self->{SAM_ROOT_URL}.'service_endpoint_status.jsp?VO_name='.$vo.'&Site_name='.$sitename.'&show_details');
        my $res = $ua->request($req);
    
        if (!$res->is_success) {
            $self->error("Could not get list of critical metrics from SAM: ".$res->status_line);
            return;
        }
    
        my $parser = new XML::DOM::Parser(ErrorContext => 2);
        my $doc = $parser->parse($res->content);

        foreach my $host ($doc->getElementsByTagName("Service")) {
            my $hostname = $host->getAttribute("endpoint");

            foreach my $service ($host->getElementsByTagName("ServiceMetric")) {
                my $metricname = $service->getAttribute("name");
                
                if (!$self->{SAM_INCLUDE_ALL} && !exists $criticalMetrics->{$vo}->{$metricname}) {
                    next;
                }
                
                foreach my $url ($service->getElementsByTagName("detailed_url")) {
                    $self->{SITEDB}->addRemoteMetric($hostname, $self->_getMetricName($metricname, $vo), "SAM", $url->getFirstChild->getNodeValue(), $metricDocs->{$metricname});
                }
            }
        }

        $doc->dispose;
    }

    1;    
}

sub _getMetricName {
    my $self = shift;
    my $name = shift;
    my $vo = shift;
    
    return $name . '-' . $vo;
}

sub _getSupportedVOs {
    my $self = shift;
    my $sitename = shift;
    my $supportedVos = {};

    my $ua = LWP::UserAgent->new;
    $ua->agent("NCG::RemoteMetrics::SAM");

    my $req = HTTP::Request->new(POST => $self->{SAM_ROOT_URL}.'services_per_vo.jsp?Site_name='.$sitename);
    my $res = $ua->request($req);

    if (!$res->is_success) {
        $self->error("Could not get supported VOs from SAM: ".$res->status_line);
        return;
    }

    my $parser = new XML::DOM::Parser(ErrorContext => 2);
    my $doc = $parser->parse($res->content);

    foreach my $vo ($doc->getElementsByTagName("supportsVO")) {
        my $voname = $vo->getFirstChild->getNodeValue();
        $supportedVos->{$voname} = 1;
    }

    $doc->dispose;

    keys %$supportedVos;
}

sub _getServices {
    my $self = shift;
    my @services;
    
    my $ua = LWP::UserAgent->new;
    $ua->agent("NCG::RemoteMetrics::SAM");

    my $req = HTTP::Request->new(POST => $self->{SAM_ROOT_URL}.'service_types.jsp');
    my $res = $ua->request($req);

    if (!$res->is_success) {
        $self->error("Could not get list of services from SAM: ".$res->status_line);
        return;
    }

    my $parser = new XML::DOM::Parser(ErrorContext => 2);
    my $doc = $parser->parse($res->content);

    foreach my $service ($doc->getElementsByTagName("Service")) {
        push @services, $service->getAttribute("type");
    }

    $doc->dispose;

    @services;    
}

sub _getCriticalMetrics {
    my $self = shift;
    my $supportedVos = shift;
    my @services = $self->_getServices;
    my $criticalMetrics = {};
    my $metrics = {};
    
    unless (@services) {
        return;
    }
    
    my $servicesStr = '&Service_type=' . join ('&Service_type=', @services );
    
    my $ua = LWP::UserAgent->new;
    $ua->agent("NCG::RemoteMetrics::SAM");

    foreach my $vo (@$supportedVos) {
        my $req = HTTP::Request->new(POST => $self->{SAM_ROOT_URL}.'servicemetrics_per_service_type_critical_for_vo.jsp?VO_name='.$vo.$servicesStr);
        my $res = $ua->request($req);
    
        if (!$res->is_success) {
            $self->error("Could not get list of critical metrics from SAM: ".$res->status_line);
            return;
        }
    
        my $parser = new XML::DOM::Parser(ErrorContext => 2);
        my $doc = $parser->parse($res->content);
    
        foreach my $service ($doc->getElementsByTagName("ServiceMetric")) {
            my $name = $service->getAttribute("name");
            $criticalMetrics->{$vo}->{$name} = 1;
            
            foreach my $url ($service->getElementsByTagName("documentation")) {
                $metrics->{$name} = $url->getFirstChild->getNodeValue();
            }
        }
    
        $doc->dispose;
    }

    return ($criticalMetrics, $metrics);
}

sub _getMetricDocUrls {
    my $self = shift;
    my $metrics = {};

    my $ua = LWP::UserAgent->new;
    $ua->agent("NCG::RemoteMetrics::SAM");

    my $req = HTTP::Request->new(POST => $self->{SAM_ROOT_URL}.'servicemetrics_per_service_type.jsp');
    my $res = $ua->request($req);

    if (!$res->is_success) {
        $self->error("Could not get list of metrics per service from SAM: ".$res->status_line);
        return;
    }

    my $parser = new XML::DOM::Parser(ErrorContext => 2);
    my $doc = $parser->parse($res->content);

    foreach my $service ($doc->getElementsByTagName("ServiceMetric")) {
        my $name = $service->getAttribute("name");
        foreach my $url ($service->getElementsByTagName("documentation")) {
            $metrics->{$name} = $url->getFirstChild->getNodeValue();
        }
    }

    $doc->dispose;

    $metrics;
}

=head1 NAME

NCG::RemoteMetrics::SAM

=head1 DESCRIPTION

The NCG::RemoteMetrics::SAM module extends NCG::RemoteMetrics module.
Module extracts list of remote metric from SAM.

=head1 SYNOPSIS

  use NCG::RemoteMetrics::SAM;

  my $ncg = NCG::RemoteMetrics::SAM->new( { SITEDB=> $sitedb } );

  $ncg->getData();

=cut

=head1 METHODS

=over

=item C<new>

  $ncg = NCG::RemoteMetrics::SAM->new( $options );

Creates new NCG::RemoteMetrics::SAM instance. Argument $options is hash
reference that can contain following elements:
  SAM_INCLUDE_ALL - if defined component will gather all tests.
  Otherwise only only critical tests are gathered.
  (default: )

  SAM_ROOT_URL - root URL used for SAM DB query interface
  (default: http://lcg-sam.cern.ch:8080/same-pi/)

  SAM_VOS - list of VOs for which SAM results will be gathered.
  If not defined all supported VOS on site are used. List of 
  supported VOs is fetched from SAM PI.
  Format: VO1[,VO]...
  (default: )

=item C<SAMRootUrl>

  print $ncg->SAMRootUrl;
  $ncg->SAMRootUrl('http://sam.server.com/new-api/');
  
Accessor method for property SAM_ROOT_URL.  

=item C<SAMVOs>

  print $ncg->SAMVOs;
  $ncg->SAMVOs('OPS,CMS');

Accessor method for property SAM_VOS.

=item C<SAMIncludeAll>

  print $ncg->SAMIncludeAll;
  $ncg->SAMIncludeAll(1);

Accessor method for property SAM_INCLUDE_ALL.

=item C<defaultSAMRootUrl>

  print $ncg->SAMRootUrl;

Read only accessor method for default value of property
SAM_ROOT_URL.

=back

=head1 SEE ALSO

NCG::RemoteMetrics

=cut

1;
