#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::SiteInfo::File;

use NCG::SiteInfo;
use strict;
use vars qw(@ISA);

@ISA=("NCG::SiteInfo");

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);

    if (! defined $self->{DB_FILE} ) {
        $self->error("File name must be defined!");
        undef $self;
        return 0;
    }

    if (! -f $self->{DB_FILE})
    {
        $self->error("Can't find static file!");
        undef $self;
        return 0;
    }
    
    $self;
}

sub getData
{
    my $self = shift;
    my $line;
    my $fileHndl;
    
    if (!open ($fileHndl, $self->{DB_FILE})) {
        $self->error("Cannot open static file!");
        return 0;
    }
    while ($line = <$fileHndl>) {
        next if ($line =~ /^\s*#.*$/);
        if ($line =~ /^\s*(\S+?)!(\S+?)!(\S+?)(!(\S+))?\s*$/) {
            my $action = $1;
            if ($action eq "HOST_SERVICE_VO") {
                next if (!defined $5);
                $self->{SITEDB}->addHost($2);
                $self->{SITEDB}->addService($2, $3);
                $self->{SITEDB}->addVO($2, $3, lc($5));
            } elsif ($action eq "HOST_SERVICE") {
                $self->{SITEDB}->addHost($2);
                $self->{SITEDB}->addService($2, $3);
            } else {
                $self->warning("Unknown command found: $1");
            }
        }
    }
    close ($fileHndl);
    
    1;
}

=head1 NAME

NCG::SiteInfo::File

=head1 DESCRIPTION

The NCG::SiteInfo::File module extends NCG::SiteInfo module. Module
extracts site information from file with list of tuples:
  # this is a comment
  # adds service for defined VO to host
  HOST_SERVICE_VO!host!service!VO
  # adds non-VO-dependent service to host
  HOST_SERVICE!host!service


=head1 SYNOPSIS

  use NCG::SiteInfo::File;

  my $siteInfo = NCG::SiteInfo::File->new( { DB_FILE => '/path/to/tuplelist' } );
  
  $siteInfo->getData();

=cut

=head1 METHODS

=over

=item C<new>

  $siteInfo = NCG::SiteInfo::File->new( $attr );

Creates new NCG::SiteInfo::File instance. Argument $attr is hash
reference that can contains following elements:
  DB_FILE - location of file with list of tuples (no default)

=back

=head1 SEE ALSO

NCG::Site

=cut

1;
