#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::SiteInfo::LDAP;

use NCG::SiteInfo;
use strict;
use Net::LDAP;
use vars qw(@ISA);

@ISA=("NCG::SiteInfo");

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);
    
    # set default values
    if (! $self->{LDAP_ADDRESS}) {
        if ( ! ($self->{LDAP_ADDRESS} = $self->{SITEDB}->siteLDAP)) {
            $self->error("LDAP_ADDRESS must be defined!");
            undef $self;
            return;
        }
    }
    # default LDAP port is BDII
    if (! defined $self->{LDAP_PORT} ) {
        $self->{LDAP_PORT} = 2170;
    }
    
    # default LDAP port is BDII
    if (! defined $self->{ADD_HOSTS} ) {
        $self->{ADD_HOSTS} = 1;
    }
    
    $self;
}

sub getContact {
    my $self = shift;
    my $sitename = shift || $self->{SITENAME} || "";
    
    if (! $self->{LDAP_BASEDN} ) {
        if ($self->{LDAP_PORT} == 2135) {
            $self->{LDAP_BASEDN} = "Mds-Vo-Name=local, O=Grid";
        } elsif ($self->{LDAP_PORT} == 2170) {
            $self->{LDAP_BASEDN} = "Mds-Vo-Name=$sitename, O=Grid";
        } else {
            $self->warning("Base DN not defined, port ". $self->{LDAP_PORT} ." is not BDII or MDS. Assuming 'O=Grid'");
            $self->{LDAP_BASEDN} = "O=Grid";
        }
    }

    my $ldap = Net::LDAP->new( $self->{LDAP_ADDRESS}, port => $self->{LDAP_PORT}, timeout => $self->{TIMEOUT}, async => 1 );
    if (! defined $ldap ) {
        $self->error("Cannot connect to $self->{LDAP_ADDRESS}:$self->{LDAP_PORT}");
        return 0;
    }
    my $mesg = $ldap->bind( anonymous => 1 );
    if ($mesg->code) {
        $self->error("Could not bind to LDAP: ".$mesg->error);
        return 0;
    }
    
    # Get site contact address
    if (! defined $self->{SITEDB}->contact()) {
        $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                               filter => "(objectClass=GlueSite)",
                               attrs=> ['GlueSiteSysAdminContact'],
                               timelimit => 60 );
        $self->{SITEDB}->addContact($1) if (!$mesg->code && $mesg->count != 0 && $mesg->entry(0)->get_value('GlueSiteSysAdminContact') =~ /^mailto:\s*(\S+)/);
    }    
}

sub getData {
    my $self = shift;
    my $sitename = shift || $self->{SITENAME} || "";

    #if ($sitename) {
    #    $self->error("Sitename must be defined!");
    #    return 0;
    #}

    my $basedn;
    my $mesg;
    my $hostname;
    my $port;
    my $entry;
    my $voEntry;

    if (! $self->{LDAP_BASEDN} ) {
        if ($self->{LDAP_PORT} == 2135) {
            $self->{LDAP_BASEDN} = "Mds-Vo-Name=local, O=Grid";
        } elsif ($self->{LDAP_PORT} == 2170) {
            $self->{LDAP_BASEDN} = "Mds-Vo-Name=$sitename, O=Grid";
        } else {
            $self->warning("Base DN not defined, port ". $self->{LDAP_PORT} ." is not BDII or MDS. Assuming 'O=Grid'");
            $self->{LDAP_BASEDN} = "O=Grid";
        }
    }

    my $ldap = Net::LDAP->new( $self->{LDAP_ADDRESS}, port => $self->{LDAP_PORT}, timeout => $self->{TIMEOUT}, async => 1 );
    if (! defined $ldap ) {
        $self->error("Cannot connect to $self->{LDAP_ADDRESS}:$self->{LDAP_PORT}");
        return 0;
    }    
    $mesg = $ldap->bind( anonymous => 1 );
    if ($mesg->code) {
        $self->error("Could not bind to LDAP: ".$mesg->error);
        return 0;
    }

    # Set site contact address
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                           filter => "(objectClass=GlueSite)",
                           attrs=> ['GlueSiteSysAdminContact'],
                           timelimit => 60 );
    $self->{SITEDB}->addContact($1) if (!$mesg->code && $mesg->count != 0 && $mesg->entry(0)->get_value('GlueSiteSysAdminContact') =~ /^mailto:\s*(\S+)/);

    # Get SE information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(objectClass=GlueSE)",
                            attrs=> ['GlueSEUniqueID'],
                            timelimit => 60 );
    
    if ($mesg->code) {
        $self->error("Could not fetch SE info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            $hostname = $entry->get_value('GlueSEUniqueID');
            
            $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
            $self->{SITEDB}->addService($hostname, "SE");

            # Get remote path that selected VO can use
            my $mesg1 = $ldap->search ( base => $self->{LDAP_BASEDN},
                                        filter => "(&(objectClass=GlueSA)(GlueChunkKey=GlueSEUniqueID=$hostname))",
                                        attrs => ['GlueSAAccessControlBaseRule'],
                                        timelimit => 60 );
            if ($mesg1->code) {
                $self->error("Could not fetch SE VO info from LDAP: ".$mesg->error);    
            } else {
                foreach my $voEntry ($mesg1->entries()) {
                    my $voname = lc($voEntry->get_value('GlueSAAccessControlBaseRule'));
                    $self->{SITEDB}->addVO($hostname, "SE", $voname);
                }
            }
        }
    }

    # Get CE information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(objectClass=GlueCE)",
                            attrs=> ['GlueCEUniqueID','GlueCEAccessControlBaseRule'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch CE info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            #lxb2006.cern.ch:2119/jobmanager-lcgpbs-dteam
            #lxb2039.cern.ch:2119/blah-pbs-dteam
            if ($entry->get_value('GlueCEUniqueID') =~ /([-_.A-Za-z0-9]+):(\d+)\/(jobmanager|blahp)-([-_.A-Za-z0-9]+)-([-_.A-Za-z0-9]+)/) {
                $hostname = $1;
                $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
                $self->{SITEDB}->addService($hostname, "CE");
                        
                foreach my $attr ($entry->get_value('GlueCEAccessControlBaseRule')) {
                    if ($attr =~ /VO:(.*)/) {
                        my $voname = lc($1);                        
                        $self->{SITEDB}->addVO($hostname, "CE", $voname);
                    }
                }
            }
        }
    }

    # Get SRM1 information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(|(GlueServiceType=srm_v1)(GlueServiceType=srm)))",
                            attrs=> ['GlueServiceURI','GlueServiceAccessControlRule'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch SRM info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceURI') =~ /httpg:\/\/([-_.A-Za-z0-9]+):(\d+)\/srm\/managerv\d+/) {
                $hostname = $1;
                $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
                $self->{SITEDB}->addService($hostname, "SRM");

                foreach my $attr ($entry->get_value('GlueServiceAccessControlRule')) {
                    my $voname = lc($attr);                    
                    $self->{SITEDB}->addVO($hostname, "SRM", $voname);
                }
            }
        }
    }

    # Get Resource Broker information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=ResourceBroker))",
                            attrs=> ['GlueServiceEndpoint','GlueServiceAccessControlRule'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch RB info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceEndpoint') =~ /([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
                $self->{SITEDB}->addService($hostname, "RB");
                
                foreach my $attr ($entry->get_value('GlueServiceAccessControlRule')) {
                    my $voname = lc($attr);                    
                    $self->{SITEDB}->addVO($hostname, "RB", $voname);
                }
            }
        }
    }

    # Get Resource Broker information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=org.glite.wms))",
                            attrs=> ['GlueServiceUniqueID','GlueServiceAccessControlRule'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch WMS info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceUniqueID') =~ /([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
                $self->{SITEDB}->addService($hostname, "WMS");

                foreach my $attr ($entry->get_value('GlueServiceAccessControlRule')) {
                    my $voname = lc($attr);
                    $self->{SITEDB}->addVO($hostname, "WMS", $voname);
                }
            }
        }
    }

    # Get Resource Broker information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=VOBOX))",
                            attrs=> ['GlueServiceUniqueID','GlueServiceAccessControlRule'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch VOBOX info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceUniqueID') =~ /gsissh:\/\/([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
                $self->{SITEDB}->addService($hostname, "VOBOX");

                foreach my $attr ($entry->get_value('GlueServiceAccessControlRule')) {
                    my $voname = lc($attr);
                    $self->{SITEDB}->addVO($hostname, "VOBOX", $voname);
                }
            }
        }
    }

    # Get MyProxy information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=MyProxy))",
                            attrs=> ['GlueServiceEndpoint'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch PROX info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceEndpoint') =~ /([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
                $self->{SITEDB}->addService($hostname, "PROX");
            }
        }
    }

    # Get MON boxes information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=gridice))",
                            attrs=> ['GlueServiceUniqueID'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch MON info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceUniqueID') =~ /([-_.A-Za-z0-9]+):(\d+)/) {
                $hostname = $1;
                $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
                $self->{SITEDB}->addService($hostname, "MON");
            }
        }
    }
    
    # Get TOP BDII information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=bdii_top))",
                            attrs=> ['GlueServiceUniqueID'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch TOP BDII info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceUniqueID') =~ /([-_.A-Za-z0-9]+):(\d+)\/bdii\-top/) {
                $hostname = $1;
                $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
                $self->{SITEDB}->addService($hostname, "BDII");
            }
        }
    }
    
    # Get Site BDII information
    $mesg = $ldap->search ( base => $self->{LDAP_BASEDN},
                            filter => "(&(objectClass=GlueService)(GlueServiceType=bdii_site))",
                            attrs=> ['GlueServiceUniqueID'],
                            timelimit => 60 );
    if ($mesg->code) {
        $self->error("Could not fetch TOP BDII info from LDAP: ".$mesg->error);
    } else {
        foreach $entry ($mesg->entries()) {
            if ($entry->get_value('GlueServiceUniqueID') =~ /([-_.A-Za-z0-9]+):(\d+)\/bdii\-site/) {
                $hostname = $1;
                $self->{SITEDB}->addHost($hostname) if ($self->{ADD_HOSTS});
                $self->{SITEDB}->addService($hostname, 'Site-BDII');
            }
        }
    }

    
    1;
}

sub baseDN {
    my $self = shift;
    my $value = shift;
    $self->{LDAP_BASEDN} = $value if (defined $value);
    $self->{LDAP_BASEDN};
}

sub address {
    my $self = shift;
    my $value = shift;
    $self->{LDAP_ADDRESS} = $value if (defined $value);
    $self->{LDAP_ADDRESS};
}

sub port {
    my $self = shift;
    my $value = shift;
    $self->{LDAP_PORT} = $value if (defined $value);
    $self->{LDAP_PORT};
}

sub addHosts {
    my $self = shift;
    my $value = shift;
    $self->{ADD_HOSTS} = $value if (defined $value);
    $self->{ADD_HOSTS};
}

=head1 NAME

NCG::SiteInfo::LDAP

=head1 DESCRIPTION

The NCG::SiteInfo::LDAP module extends NCG::SiteInfo module. Module
extracts site information from LDAP-based information services (e.g.
BDII, Globus MDS).

Module also sets site-level email contact address based on 
attribute GlueSiteSysAdminContact.

=head1 SYNOPSIS

  use NCG::SiteInfo::LDAP;

  my $siteInfo = NCG::SiteInfo::LDAP->new({LDAP_ADDRESS=>'ldap.server.com', 
                                           LDAP_PORT=>2170, 
                                           LDAP_BASEDN=>'O=Grid'});

  $siteInfo->getData();

  $siteInfo->getContact();

=cut

=head1 METHODS

=over

=item C<new>

  my $siteInfo = NCG::SiteInfo::LDAP->new($options); 

Creates new NCG::SiteInfo::LDAP instance. Argument $options is hash reference that
can contains following elements:
  LDAP_ADDRESS - address of LDAP server (default: localhost)
  LDAP_PORT - port of LDAP server (default: 2170)
  LDAP_BASEDN - basedn used when querying LDAP server
  ADD_HOSTS - if true hosts found in LDAP will be added, otherwise only service on existing hosts are added (default: true)

=item C<address>

  print $siteInfo->address;
  $siteInfo->address('ldap.server.com');

Accessor method for property address.

=item C<port>

  print $siteInfo->port;
  $siteInfo->port(2135);

Accessor method for property port.

=item C<baseDN>

  print $siteInfo->baseDN;
  $siteInfo->baseDN('Mds-Vo-Name=myegeesite,O=Grid');

Accessor method for property baseDN.

=item C<addHosts>

  print $siteInfo->addHosts;
  $siteInfo->addHosts(0);

Accessor method for property ADD_HOSTS.

=item C<getContact>

  print $siteInfo->getContact;

Method retrieves site contact information from BDII.
(GlueSiteSysAdminContact)

=back

=head1 SEE ALSO

NCG::SiteInfo

=cut

1;
