#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package NCG::SiteInfo::SAM;

use strict;
use NCG::SiteInfo;
use LWP::UserAgent;
use XML::DOM;

use vars qw(@ISA);

@ISA=("NCG::SiteInfo");

my $DEFAULT_SAM_ROOT_URL = "http://lcg-sam.cern.ch:8080/same-pi/";

sub new
{
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self =  $class->SUPER::new(@_);

    # set default values
    if (! $self->{SAM_ROOT_URL}) {
        $self->{SAM_ROOT_URL} = $DEFAULT_SAM_ROOT_URL;
    }

    $self;
}

sub SAMRootUrl {
    my $self = shift;
    my $value = shift;
    $self->{SAM_ROOT_URL} = $value if (defined $value);
    $self->{SAM_ROOT_URL};
}

sub defaultSAMRootUrl {
    my $self = shift;
    return $DEFAULT_SAM_ROOT_URL;
}

sub _transformSAMName ($)
{
    my $self = shift;
    my $samname = shift;
    if ($samname eq 'RGMA') {
        $samname = 'MON';
    } elsif ($samname eq 'MyProxy') {
        $samname = 'PROX';
    } elsif ($samname eq 'sBDII') {
        $samname = 'Site-BDII';
    } elsif ($samname eq 'gCE') {
        $samname = 'gLite-CE';
    } elsif ($samname eq 'gRB') {
        $samname = 'WMS';
    } elsif ($samname eq 'SRMv2') {
        $samname = 'SRM';
    }

    $samname;
}

sub getData
{
    my $self = shift;
    my $sitename = shift || $self->{SITENAME};;

    my $ua = LWP::UserAgent->new;
    $ua->agent("NCG::SiteInfo::SAM");

    my $req = HTTP::Request->new(POST => $self->{SAM_ROOT_URL} . 'service_types_per_service_endpoint.jsp?Site_name='.$sitename);
    my $res = $ua->request($req);

    if (!$res->is_success) {
        $self->error("Could not get results from SAM: ".$res->status_line);
        return 0;
    }

    my $parser = new XML::DOM::Parser(ErrorContext => 2);
    my $doc = $parser->parse($res->content);

    foreach my $host ($doc->getElementsByTagName("Service")) {
        my $hostname = $host->getAttribute("endpoint");
        my $service = $self->_transformSAMName($host->getAttribute("type"));

        $self->{SITEDB}->addHost($hostname);
        $self->{SITEDB}->addService($hostname, $service);
        
        $self->{SITEDB}->siteLDAP($hostname) if ($service eq 'Site-BDII');
    }

    $doc->dispose;

    $req = HTTP::Request->new(POST => $self->{SAM_ROOT_URL} . 'services_per_vo.jsp?Site_name='.$sitename);
    $res = $ua->request($req);

    if (!$res->is_success) {
        $self->setError("Could not get results from SAM: ".$res->status_line);
        return 0;
    }
    
    $parser = new XML::DOM::Parser(ErrorContext => 2);
    $doc = $parser->parse($res->content);
    
    foreach my $host ($doc->getElementsByTagName("Service")) {
        my $hostname = $host->getAttribute("endpoint");
        my $service = $self->_transformSAMName($host->getAttribute("type"));
        
        $self->{SITEDB}->addHost($hostname);
        $self->{SITEDB}->addService($hostname, $service);

        # dirty hack - SAM reports only OPS VO for BDII
        next if ($service eq "BDII" || $service eq "Site-BDII");

        foreach my $vo ($host->getElementsByTagName("supportsVO")) {
            my $voname = lc($vo->getFirstChild->getNodeValue());
            $self->{SITEDB}->addVO($hostname, $service, $voname);
        }
    }
    
    $doc->dispose;

    1;
}

=head1 NAME

NCG::SiteInfo::SAM

=head1 DESCRIPTION

The NCG::SiteInfo::SAM module extends NCG::SiteInfo module. Module
extracts site information from SAM (eng. Service Availability Monitoring)
database.

=head1 SYNOPSIS

  use NCG::SiteInfo::SAM;

  my $siteInfo = NCG::SiteInfo::SAM->new({ { SITEDB=> $sitedb } );

  $siteInfo->getData();

=cut

=head1 METHODS

=over

=item C<new>

  $siteInfo = NCG::SiteInfo::SAM->new( $options );

Creates new NCG::SiteInfo::SAM instance. Argument $options is hash
reference that can contain following elements:
  SAM_ROOT_URL - root URL used for SAM DB query interface
  (default: http://lcg-sam.cern.ch:8080/same-pi/)

=item C<SAMRootUrl>

  print $ncg->SAMRootUrl;
  $ncg->SAMRootUrl('http://sam.server.com/new-api/');
  
Accessor method for property SAM_ROOT_URL.  

=item C<defaultSAMRootUrl>

  print $ncg->SAMRootUrl;

Read only accessor method for default value of property
SAM_ROOT_URL.

=back

=head1 SEE ALSO

NCG::Site

=cut

1;
