#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package SiteDB;
use strict;
use Socket;

sub new {
    my $proto  = shift;
    my $class  = ref($proto) || $proto;
    my $self = shift || {};
    bless ($self, $class);
    $self;
}

sub verbose {
    my $self = shift;
    my $message = shift;
    print "$message\n" if ($self->{VERBOSE});
}

sub warning($) {
    my $self = shift;
    my $msg = shift;
    print "WARNING: $msg\n" if ($self->{WARNING});
    1;
}

sub error {
    my $self = shift;
    my $message = shift;
    print "SITEDB ERROR: $message\n" if ($self->{ERROR});
}

sub debug {
    my $self = shift;
    my $message = shift;
    print "DEBUG: $message\n" if ($self->{DEBUG});
}

sub debugSub {
    my $self = shift;
    return 1 if (!$self->{DEBUG});

    my $sub = ( caller(1) )[3];
    my $args = "";
    foreach my $arg (@_) {
        if ($arg) {
            $args .= "$arg ";
        } else {
            $args .= "undef ";
        }
    }
    chop $args;
    print "DEBUG: in $sub with args: $args\n";
    
    1;
}

# Add methods

sub _addHostReal {
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift;
    my $address;
    my ($hostReal, $hostname1, $aliases, $addrtype, $length, @addrs, @addrs1);

    # get detailed information about this host &
    # check if host exists at all
    unless ( ($hostReal, $aliases, $addrtype, $length, @addrs) = gethostbyname($host)) {
        return;
    }

    $self->{HOSTS}->{$host} = {};
    $self->{HOSTS}->{$host}->{REALHOST} = $hostReal;

    # add host to list of real hosts
    $self->{REALHOSTS}->{$hostReal} = {} if (!exists $self->{REALHOSTS}->{$hostReal});

    # we're dealing with alias here
    if ($hostReal ne $host) {
        $self->{HOSTS}->{$host}->{ADDRESS} = $host;
        $self->{HOSTS}->{$host}->{ALIAS} = $hostReal;
                
        # Add real host
        $self->{REALHOSTS}->{$hostReal}->{ALIASES}->{$host} = {};
        
        $self->verbose ("Found alias: $host, real hostname: $hostReal");
    } else {
        # here we use hostname for address field only if host has 
        # load-balanced nodes
        $self->verbose ("Found host: $host.");
        if ($#addrs) {
            $self->{HOSTS}->{$host}->{ADDRESS} = $host;
        } else {
            $self->{HOSTS}->{$host}->{ADDRESS} = join ('.', unpack('C4', $addrs[0]));
        }
    }    
    
    # if there are load-balanced nodes add them to list
    # of aliases, use their IP addresses for address field
    if ($#addrs && ! exists $self->{REALHOSTS}->{$hostReal}->{LBNODES}) {
        foreach my $addr (@addrs) {
            if (($hostname1, $aliases, $addrtype, $length, @addrs1) = gethostbyaddr($addr, AF_INET)) {
                $self->{REALHOSTS}->{$hostReal}->{LBNODES}->{$hostname1}->{ADDRESS} = join ('.', unpack('C4', $addr));
                $self->verbose("  Found LB node: $hostname1.");
            }
        }
    }

    1;
}

sub addHost
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $res;
    
    $self->_addHostReal($host) if (! exists $self->{HOSTS}->{$host});
}

sub addService
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $service = shift || return;

    # host is in HOSTS, add service to REALHOSTS
    if (exists $self->{HOSTS}->{$host}) {
        if (! exists $self->{HOSTS}->{$host}->{SERVICES}->{$service}) {
            $self->{HOSTS}->{$host}->{SERVICES}->{$service} = {};
        }
        
        my $realHost = $self->{HOSTS}->{$host}->{REALHOST};
        if (! exists $self->{REALHOSTS}->{$realHost}->{SERVICES}->{$service}) {
            $self->{REALHOSTS}->{$realHost}->{SERVICES}->{$service} = {};
        }
    }
    # host is only in REALHOSTS, add service to REALHOSTS & to all ALIASES
    # TODO:
    #  what if the alias is added afterwards? (this should probably be handled
    #  afterwards in consistency check)
    elsif (exists $self->{REALHOSTS}->{$host}) {
        if (! exists $self->{REALHOSTS}->{$host}->{SERVICES}->{$service}) {
            $self->{REALHOSTS}->{$host}->{SERVICES}->{$service} = {};
        }
        if ( exists $self->{REALHOSTS}->{$host}->{ALIASES}) {
            foreach my $hostname (keys %{$self->{REALHOSTS}->{$host}->{ALIASES}}) {
                if (! exists $self->{HOSTS}->{$hostname}->{SERVICES}->{$service}) {
                    $self->{HOSTS}->{$hostname}->{SERVICES}->{$service} = {};
                }
            }
        }
    }
    # host doesn't exist, this is not allowed
    else {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }

    1;
}

sub addVO
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $service = shift || return;
    my $vo = shift;
    
    # host is in HOSTS, add service to REALHOSTS
    if (exists $self->{HOSTS}->{$host}) {
        # if service is not on the host return error
        if (! exists $self->{HOSTS}->{$host}->{SERVICES}->{$service} ) {
            $self->warning ("Service $service is not present on host $host!");
            return;
        }
        
        $self->{HOSTS}->{$host}->{SERVICES}->{$service}->{VOS}->{$vo} = 0;
        
        my $realHost = $self->{HOSTS}->{$host}->{REALHOST};
        $self->{REALHOSTS}->{$realHost}->{SERVICES}->{$service}->{VOS}->{$vo} = 0;
    }
    # host is only in REALHOSTS, add VO to REALHOSTS & to all ALIASES that have service
    # TODO:
    #  what if the alias is added afterwards? (this should probably be handled
    #  afterwards in consistency check)
    elsif (exists $self->{REALHOSTS}->{$host}) {
        if (! exists $self->{REALHOSTS}->{$host}->{SERVICES}->{$service} ) {
            $self->warning ("Service $service is not present on host $host!");
            return;
        }
        
        $self->{REALHOSTS}->{$host}->{SERVICES}->{$service}->{VOS}->{$vo} = 0;

        if ( exists $self->{REALHOSTS}->{$host}->{ALIASES}) {
            foreach my $hostname (keys %{$self->{REALHOSTS}->{$host}->{ALIASES}}) {
                if ( exists $self->{HOSTS}->{$hostname}->{SERVICES}->{$service}) {
                    $self->{HOSTS}->{$hostname}->{SERVICES}->{$service}->{VOS}->{$vo} = 0;
                }
            }
        }
    }
    # host doesn't exist, this is not allowed
    else {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }    

    1;
}

# metricsets are stored in both REALHOSTS and HOSTS
sub addMetricSet
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metricSet = shift || return;
    my $service = shift || return;

    # host must be present in HOSTS list 
    if (!exists $self->{HOSTS}->{$host} ) {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }
   
    # add metricset to HOSTS
    if (!exists $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet}) {
        $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet} = {};
    }
    $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet}->{SERVICES}->{$service} = 1;
    
    # add metricset to REALHOSTS
    my $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    if (!exists $self->{REALHOSTS}->{$realHost}->{METRICSETS}->{$metricSet}) {
        $self->{REALHOSTS}->{$realHost}->{METRICSETS}->{$metricSet} = {};
    }
    $self->{REALHOSTS}->{$realHost}->{METRICSETS}->{$metricSet}->{SERVICES}->{$service} = 1;
    
    1;
}

# remote metrics are not propagated to REALHOSTS
sub addRemoteMetric
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift;
    my $metric = shift;
    my $remoteService = shift;
    my $url = shift;
    my $docUrl = shift;
    
    if (!exists $self->{HOSTS}->{$host} ) {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }
    
    $self->{HOSTS}->{$host}->{METRICS}->{$metric} = {};
    $self->{HOSTS}->{$host}->{METRICS}->{$metric}->{TYPE} = "remote";
    $self->{HOSTS}->{$host}->{METRICS}->{$metric}->{REMOTESERVICE} = $remoteService;
    $self->{HOSTS}->{$host}->{METRICS}->{$metric}->{URL} = $url;
    $self->{HOSTS}->{$host}->{METRICS}->{$metric}->{DOCURL} = $docUrl;

    1;
}

# local metrics are stored in REALHOSTS; HOSTS contains only references
sub addLocalMetric
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift;
    my $metric = shift;
    my $metricSet = shift;
    my $probe = shift;
    my $native = shift;
    my $vo = shift;
    my $config = shift;
    my $attributes = shift;
    my $dependencies = shift;
    my $docUrl = shift;
    
    if (!exists $self->{HOSTS}->{$host} ) {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }
    
    my $realHost = $self->{HOSTS}->{$host}->{REALHOST};

    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric} = {};
    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{TYPE} = "local";
    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{METRICSET} = $metricSet;
    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{PROBE} = $probe;
    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{NATIVE} = $native;
    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{VO} = $vo;
    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{CONFIG} = $config;
    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{ATTRIBUTES} = $attributes;
    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{DEPENDENCIES} = $dependencies;
    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{DOCURL} = $docUrl;

    $self->{HOSTS}->{$host}->{METRICS}->{$metric} = $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric};
    
    1;
}

# LB nodes are stored only in REALHOSTS
sub addLBNode
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $lbnode = shift || return;
    my $realHost;
    my $addr;
    
    if (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } elsif (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } else {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }
    
    unless ($addr = scalar gethostbyname($lbnode)) {
        $self->error("Invalid LB node hostname: $lbnode.");
        return;
    }

    $self->{REALHOSTS}->{$realHost}->{LBNODES}->{$lbnode}->{ADDRESS} = inet_ntoa($addr);
    
    1;
}

sub addContact
{
    my $self = shift;

    $self->debugSub(@_);

    my $value = shift || return;
    
    $self->{CONTACTS}->{$value} = 1;
    
    1;
}

# Contacts are stored in REALHOSTS only
sub addHostContact
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $value = shift || return;

    if (exists $self->{CONTACTS}->{$value})
    {
        $self->warning ("Contact is already in site contacts.");
        return;
    }

    if (exists $self->{REALHOSTS}->{$host}) {
        $self->{REALHOSTS}->{$host}->{CONTACTS}->{$value} = 1;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        my $realHost = $self->{HOSTS}->{$host}->{REALHOST};
        $self->{REALHOSTS}->{$realHost}->{CONTACTS}->{$value} = 1;
    } else {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }

    1;
}

# Remove methods

# host is removed from REALHOSTS only in case it doesn't have any aliases left
sub _removeHostReal ($$$)
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift;
    my $hostAlias = shift;

    # check if host still has some aliases
    if (exists $self->{REALHOSTS}->{$host}->{ALIASES}) {
        delete $self->{REALHOSTS}->{$host}->{ALIASES}->{$hostAlias};

        if ( ! %{$self->{REALHOSTS}->{$host}->{ALIASES}} ) {
            delete $self->{REALHOSTS}->{$host};
        }
    } else {
        delete $self->{REALHOSTS}->{$host};
    }
}

sub removeHost
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;

    if (!exists $self->{HOSTS}->{$host} ) {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }

    $self->_removeHostReal($self->{HOSTS}->{$host}->{REALHOST}, $host);
    delete $self->{HOSTS}->{$host};

    1;
}

# remove metricset from REALHOST only
sub _removeMetricSetReal
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift;
    my $metricSet = shift;
    
    if ( exists $self->{REALHOSTS}->{$host}->{ALIASES}) {
        foreach my $hostname (keys %{$self->{REALHOSTS}->{$host}->{ALIASES}}) {
            delete $self->{HOSTS}->{$hostname}->{METRICSETS}->{$metricSet}; 
        }
    }

    delete $self->{REALHOSTS}->{$host}->{METRICSETS}->{$metricSet};
}

sub removeMetricSet
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metricSet = shift  || return;
    my $hostOnly = shift;
    
    if (exists $self->{HOSTS}->{$host} ) {
        delete $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet};
        $self->_removeMetricSetReal($self->{HOSTS}->{$host}->{REALHOST}, $metricSet) if (!$hostOnly);
    } elsif (exists $self->{REALHOSTS}->{$host}) {
        $self->_removeMetricSetReal($host, $metricSet);
    } else {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }

    1;
}

sub removeLBNode
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $lbnode = shift || return;
    my $realHost;
    
    if (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } elsif (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }

    delete $self->{REALHOSTS}->{$realHost}->{LBNODES}->{$lbnode};

    1;
}

# Accessor methods

sub siteName
{
    my $self = shift;

    $self->debugSub(@_);

    my $value = shift;

    $self->{SITENAME} = $value if (defined $value);
    $self->{SITENAME};
}

sub siteLDAP
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $value = shift;

    $self->{LDAP_ADDRESS} = $value if (defined $value);
    $self->{LDAP_ADDRESS};
}

# attributes are stored only in REALHOSTS
sub hostAttribute
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $attribute = shift || return;
    my $value = shift;
    my $realHost;
    
    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }
    
    $self->{REALHOSTS}->{$realHost}->{ATTRIBUTES}->{$attribute} = $value if (defined $value);
    $self->{REALHOSTS}->{$realHost}->{ATTRIBUTES}->{$attribute};
}

# attributes are stored only in REALHOSTS
sub hostAttributeVO
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift;
    my $attribute = shift;
    my $vo = shift;
    my $value = shift;
    my $realHost;
    
    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }
    
    $self->{REALHOSTS}->{$realHost}->{ATTRIBUTES}->{$attribute}->{VOS}->{$vo} = $value if (defined $value);
    $self->{REALHOSTS}->{$realHost}->{ATTRIBUTES}->{$attribute}->{VOS}->{$vo};
}

sub hostAttributeVOs
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $attribute = shift || return;
    my $realHost;

    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }

    if (! exists $self->{REALHOSTS}->{$realHost}->{ATTRIBUTES}->{$attribute}) {
        $self->warning("Attribute $attribute on host $host doesn't exist.");
        return;
    }

    return values %{$self->{REALHOSTS}->{$realHost}->{ATTRIBUTES}->{$attribute}->{VOS}};
}

sub metricSetServiceURI
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metricSet = shift || return;
    my $value = shift;
    
    # host is in HOSTS
    if (exists $self->{HOSTS}->{$host}) {
        if (! exists $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet} ) {
            $self->warning ("Metric set $metricSet is not present on host $host!");
            return;
        }

        # if value contains REALHOST set entry in REALHOSTS, modify value and set entry in HOSTS
        if (defined $value) {
            my $realHost = $self->{HOSTS}->{$host}->{REALHOST};
            if ($realHost ne $host) {
                if ($value =~ /$realHost/) {
                    $self->{REALHOSTS}->{$realHost}->{METRICSETS}->{$metricSet}->{SERVICEURI} = $value;
                    $value =~ s/$realHost/$host/g;
                } else {
                    my $valueTmp = $value;
                    $valueTmp =~ s/$host/$realHost/g;
                    $self->{REALHOSTS}->{$realHost}->{METRICSETS}->{$metricSet}->{SERVICEURI} = $valueTmp;
                }
            } else {
                $self->{REALHOSTS}->{$realHost}->{METRICSETS}->{$metricSet}->{SERVICEURI} = $value;
            }
            $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet}->{SERVICEURI} = $value;
        }

        return $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet}->{SERVICEURI};
    }
    # host is only in REALHOSTS
    elsif (exists $self->{REALHOSTS}->{$host}) {
        if (! exists $self->{REALHOSTS}->{$host}->{METRICSETS}->{$metricSet} ) {
            $self->warning ("Metric set $metricSet is not present on host $host!");
            return;
        }

        # set value in REALHOSTS and propagate modified value to HOSTS (where SERVICEURI is not defined)
        if (defined $value) {
            $self->{REALHOSTS}->{$host}->{METRICSETS}->{$metricSet}->{SERVICEURI} = $value;
            if ( exists $self->{REALHOSTS}->{$host}->{ALIASES}) {
                foreach my $hostname (keys %{$self->{REALHOSTS}->{$host}->{ALIASES}}) {
                    my $uriValue = $value;
                    if ( exists $self->{HOSTS}->{$hostname}->{METRICSETS}->{$metricSet} && ! exists $self->{HOSTS}->{$hostname}->{METRICSETS}->{$metricSet}->{SERVICEURI}) {
                        $uriValue =~ s/$host/$hostname/g;
                        $self->{HOSTS}->{$hostname}->{METRICSETS}->{$metricSet}->{SERVICEURI} = $uriValue;
                    }
                }
            }
        }

        return $self->{REALHOSTS}->{$host}->{METRICSETS}->{$metricSet}->{SERVICEURI};
    }
    # host doesn't exist, this is not allowed
    else {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }
}

sub metricSetServiceURILBNode
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $lbnode = shift || return;
    my $host = shift || return;
    my $metricSet = shift || return;
    my $realHost;
    my $value;


    # host is in HOSTS
    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host}) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    }

    if ($realHost && exists $self->{REALHOSTS}->{$realHost}->{METRICSETS}->{$metricSet}) {
        $value = $self->{REALHOSTS}->{$realHost}->{METRICSETS}->{$metricSet}->{SERVICEURI};
        if ($value) {
            $value =~ s/$realHost/$lbnode/g;
        } else {
            $value = $lbnode;
        }
    }
    
    return $value;
}

sub _metricField
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift;
    my $metric = shift;
    my $attr = shift;
    my $value = shift;

    # host is in HOSTS
    if (exists $self->{HOSTS}->{$host}) {
        #my $realHost = $self->{HOSTS}->{$host}->{REALHOST};
        if (exists $self->{HOSTS}->{$host}->{METRICS}->{$metric})
        {
            #$self->{HOSTS}->{$host}->{METRICS}->{$metric}->{$attr} = $value if (defined $value);
            return $self->{HOSTS}->{$host}->{METRICS}->{$metric}->{$attr};
        }
        #elsif (exists $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric})
        #{
        #    $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{$attr} = $value if (defined $value);
        #    return $self->{REALHOSTS}->{$realHost}->{METRICS}->{$metric}->{$attr};
        #}
        else
        {
            $self->warning ("Metric $metric is not present on host $host!");
            return;
        }
    }
    # host is only in REALHOSTS
    elsif (exists $self->{REALHOSTS}->{$host})
    {
        if (exists $self->{REALHOSTS}->{$host}->{METRICS}->{$metric})
        {
            #$self->{REALHOSTS}->{$host}->{METRICS}->{$metric}->{$attr} = $value if (defined $value);
            return $self->{REALHOSTS}->{$host}->{METRICS}->{$metric}->{$attr};
        }
        else
        {
            $self->warning ("Metric $metric is not present on host $host!");
            return;
        }
    }
    # host doesn't exist, this is not allowed
    else {
        $self->warning ("Host $host is not in the list of hosts on site!");
        return;
    }
}

sub metricType
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;
    
    $self->_metricField ($host, $metric, "TYPE", $value);
}

# metricDocUrl can belong to both remote and local service so we need to check
# both HOSTS AND REALHOSTS
sub metricDocUrl
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;
    
    $self->_metricField ($host, $metric, "DOCURL", $value);
}

sub metricProbe
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;
    
    $self->_metricField ($host, $metric, "PROBE", $value);
}

sub metricNative
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;

    $self->_metricField ($host, $metric, "NATIVE", $value);
}

sub metricVo
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;
    
    $self->_metricField ($host, $metric, "VO", $value);
}

sub metricRemoteService
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;
    
    $self->_metricField ($host, $metric, "REMOTESERVICE", $value);
}

sub metricUrl
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;
    
    $self->_metricField ($host, $metric, "URL", $value);
}

sub metricMetricSet
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;
    
    $self->_metricField ($host, $metric, "METRICSET", $value);
}

sub metricConfig
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;
    
    $self->_metricField ($host, $metric, "CONFIG", $value);
}

sub metricAttributes
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;

    $self->_metricField ($host, $metric, "ATTRIBUTES", $value);
}

sub metricDependencies
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $metric = shift || return;
    my $value = shift;
    
    $self->_metricField ($host, $metric, "DEPENDENCIES", $value);
}

# read-only
sub hostAddress
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    
    if (!exists $self->{HOSTS}->{$host} ) {
        $self->error ("Host $host is not in the list of hosts on site!");
        return;
    }
    
    $self->{HOSTS}->{$host}->{ADDRESS};    
}

sub realHost
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    
    if (!exists $self->{HOSTS}->{$host} ) {
        $self->error ("Host $host is not in the list of hosts on site!");
        return;
    }
    
    $self->{HOSTS}->{$host}->{REALHOST};    
}

sub hostAlias
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;

    if (!exists $self->{HOSTS}->{$host} ) {
        $self->error ("Host $host is not in the list of hosts on site!");
        return;
    }

    $self->{HOSTS}->{$host}->{ALIAS};
}

sub LBNodeAddress
{
    my $self = shift;
    
    $self->debugSub(@_);

    my $host = shift || return;
    my $lbnode = shift || return;
    my $realHost;

    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }
    
    if (exists $self->{REALHOSTS}->{$realHost}->{LBNODES}) {
        return $self->{REALHOSTS}->{$realHost}->{LBNODES}->{$lbnode}->{ADDRESS};
    }
}

# iterator methods

sub getHosts
{
    my $self = shift;

    $self->debugSub(@_);

    return keys %{$self->{HOSTS}};
}

sub getRealHosts
{
    my $self = shift;
    
    $self->debugSub(@_);

    return keys %{$self->{REALHOSTS}};
}

sub getServices
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    
    if (exists $self->{HOSTS}->{$host}) {
        return keys %{$self->{HOSTS}->{$host}->{SERVICES}};
    } else {
        return;
    }
}

sub getRealServices
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    
    if (exists $self->{REALHOSTS}->{$host}) {
        return keys %{$self->{REALHOSTS}->{$host}->{SERVICES}};
    } else {
        return;
    }
}

sub getVOsByHostService
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $service = shift || return;
    
    if (exists $self->{HOSTS}->{$host}) {
        if (exists $self->{HOSTS}->{$host}->{SERVICES}->{$service}) {
            return keys %{$self->{HOSTS}->{$host}->{SERVICES}->{$service}->{VOS}};
        } else {
            return;
        }
    } else {
        return;
    }
}

sub getMetricSets
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;

    if (exists $self->{HOSTS}->{$host}) {
        return keys %{$self->{HOSTS}->{$host}->{METRICSETS}};
    } else {
        return;
    }
}

sub getRealMetricSets
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;

    if (exists $self->{REALHOSTS}->{$host}) {
        return keys %{$self->{REALHOSTS}->{$host}->{METRICSETS}};
    } else {
        return;
    }
}

sub getMetrics
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    
    if (exists $self->{HOSTS}->{$host}) {
        return keys %{$self->{HOSTS}->{$host}->{METRICS}};
    } else {
        return;
    }
}

sub getLocalMetrics
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;

    if (exists $self->{HOSTS}->{$host}) {
        my @metrics;
        foreach my $metric ( keys %{$self->{HOSTS}->{$host}->{METRICS}}) {
            if ($self->{HOSTS}->{$host}->{METRICS}->{$metric}->{TYPE} eq 'local' &&
                ! defined $self->{HOSTS}->{$host}->{METRICS}->{$metric}->{NATIVE} ) {
                push @metrics, $metric;
            }
        }
        return @metrics;
    } else {
        return;
    }
}

sub getNativeMetrics
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $native = shift || return;

    if (exists $self->{HOSTS}->{$host}) {
        my @metrics;
        foreach my $metric ( keys %{$self->{HOSTS}->{$host}->{METRICS}}) {
            if ($self->{HOSTS}->{$host}->{METRICS}->{$metric}->{TYPE} eq 'local' &&
                defined $self->{HOSTS}->{$host}->{METRICS}->{$metric}->{NATIVE} &&
                $self->{HOSTS}->{$host}->{METRICS}->{$metric}->{NATIVE} eq $native) {
                push @metrics, $metric;
            }
        }
        return @metrics;
    } else {
        return;
    }
}

sub getRemoteMetrics
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;

    if (exists $self->{HOSTS}->{$host}) {
        my @metrics;
        foreach my $metric ( keys %{$self->{HOSTS}->{$host}->{METRICS}}) {
            if ($self->{HOSTS}->{$host}->{METRICS}->{$metric}->{TYPE} eq 'remote') {
                push @metrics, $metric;
            }
        }
        return @metrics;
    } else {
        return;
    }
}

sub getRealMetrics
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    
    if (exists $self->{REALHOSTS}->{$host}) {
        return keys %{$self->{REALHOSTS}->{$host}->{METRICS}};
    } else {
        return;
    }
}

sub getRealLocalMetrics
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;

    if (exists $self->{REALHOSTS}->{$host}) {
        my @metrics;
        foreach my $metric ( keys %{$self->{REALHOSTS}->{$host}->{METRICS}}) {
            if ($self->{REALHOSTS}->{$host}->{METRICS}->{$metric}->{TYPE} eq 'local' &&
                ! defined $self->{REALHOSTS}->{$host}->{METRICS}->{$metric}->{NATIVE} ) {
                push @metrics, $metric;
            }
        }
        return @metrics;
    } else {
        return;
    }
}

sub getRealNativeMetrics
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $native = shift || return;

    if (exists $self->{REALHOSTS}->{$host}) {
        my @metrics;
        foreach my $metric ( keys %{$self->{REALHOSTS}->{$host}->{METRICS}}) {
            if ($self->{REALHOSTS}->{$host}->{METRICS}->{$metric}->{TYPE} eq 'local' &&
                defined $self->{REALHOSTS}->{$host}->{METRICS}->{$metric}->{NATIVE} &&
                $self->{REALHOSTS}->{$host}->{METRICS}->{$metric}->{NATIVE} eq $native) {
                push @metrics, $metric;
            }
        }
        return @metrics;
    } else {
        return;
    }
}

sub getLBNodes
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $realHost;

    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }
    
    if (exists $self->{REALHOSTS}->{$realHost}->{LBNODES}) {
        return keys %{$self->{REALHOSTS}->{$realHost}->{LBNODES}};
    }
}

sub getAliases
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $realHost;

    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }
    
    if (exists $self->{REALHOSTS}->{$realHost}->{ALIASES}) {
        return keys %{$self->{REALHOSTS}->{$realHost}->{ALIASES}};
    }
}

sub getContacts
{
    my $self = shift;

    $self->debugSub(@_);

    if (exists $self->{CONTACTS}) {
        return keys %{$self->{CONTACTS}};
    }
}

sub getHostContacts
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $realHost;

    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }

    if (exists $self->{REALHOSTS}->{$realHost}->{CONTACTS}) {
        return keys %{$self->{REALHOSTS}->{$realHost}->{CONTACTS}};
    }
}

# check methods

sub hasHost
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $hosts;
    
    return exists $self->{HOSTS}->{$host};
}

sub hasRealHost
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $hosts;
    
    return exists $self->{REALHOSTS}->{$host};

}

sub hasService
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $service = shift || return;
    
    if (exists $self->{HOSTS}->{$host} ) {
        return exists $self->{HOSTS}->{$host}->{SERVICES}->{$service};
    }
}

sub hasRealService
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $service = shift || return;

    if (exists $self->{REALHOSTS}->{$host}) {
        return exists $self->{REALHOSTS}->{$host}->{SERVICES}->{$service};
    }
}

sub hasMetricSet
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $metricSet = shift || return;
    
    if (exists $self->{HOSTS}->{$host} ) {
        return exists $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet};
    }
}

sub hasRealMetricSet
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $metricSet = shift || return;
    
    if (exists $self->{REALHOSTS}->{$host}) {
        return exists $self->{REALHOSTS}->{$host}->{METRICSETS}->{$metricSet};
    }
}

sub hasAttribute
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $attr = shift || return;
    my $realHost;
    
    if (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } elsif (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } else {
        $self->error("Host $host is not in the list of hosts on site!");
        return;
    }
    
    return exists $self->{REALHOSTS}->{$realHost}->{ATTRIBUTES}->{$attr};
}

sub hasServiceURI
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $metricSet = shift || return;
    
    # host is in HOSTS
    if (exists $self->{HOSTS}->{$host}) {
        if (! exists $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet} ) {
            $self->error ("Metric set $metricSet is not present on host $host!");
            return;
        }
        
        return exists $self->{HOSTS}->{$host}->{METRICSETS}->{$metricSet}->{SERVICEURI};
    }
    # host is only in REALHOSTS
    elsif (exists $self->{REALHOSTS}->{$host}) {
        if (! exists $self->{REALHOSTS}->{$host}->{METRICSETS}->{$metricSet} ) {
            $self->error ("Metric set $metricSet is not present on host $host!");
            return;
        }

        return exists $self->{REALHOSTS}->{$host}->{METRICSETS}->{$metricSet}->{SERVICEURI};
    }
    # host doesn't exist, this is not allowed
    else {
        $self->error ("Host $host is not in the list of hosts on site!");
        return;
    }
}

sub hasContacts
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $realHost;

    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }

    return (exists $self->{REALHOSTS}->{$realHost}->{CONTACTS});
}

sub hasLBNodes
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $realHost;

    if (exists $self->{REALHOSTS}->{$host}) {
        $realHost = $host;
    } elsif (exists $self->{HOSTS}->{$host} ) {
        $realHost = $self->{HOSTS}->{$host}->{REALHOST};
    } else {
        $self->warning("Host $host is not in the list of hosts on site!");
        return;
    }
    
    return (exists $self->{REALHOSTS}->{$realHost}->{LBNODES});
}


# TODO:
#   - temporary solution: this needs to go per metricset
#   - issues: what if metricset is shared between
sub hasVO
{
    my $self = shift;

    $self->debugSub(@_);

    my $host = shift || return;
    my $vo = shift || return;
    my $result;
    my $hostServices;
    $vo = lc($vo);

    if (exists $self->{HOSTS}->{$host}) {
        $hostServices = $self->{HOSTS}->{$host}->{SERVICES};
    } elsif (exists $self->{REALHOSTS}->{$host}) {
        $hostServices = $self->{REALHOSTS}->{$host}->{SERVICES};
    }
    
    if (defined $hostServices) {
        # if services have no VO assigned we assume that VO is supported
        # examples are: MON, BDII, Site-BDII
        $result = 1;
        foreach my $service (keys %$hostServices) {
            if (exists $hostServices->{$service}->{VOS}) {
                if (exists $hostServices->{$service}->{VOS}->{$vo}) {
                    $result = 1;
                    last;
                } else {
                    $result = 0;
                    last;
                }
            }
        }
        return $result;
    }
}


# consistency check

#sub _checkHostDependencies
#{
#    my $self = shift;
    
#    foreach my $metric (
#}

# iterate through REALHOSTS and clean up metrics with unsatisfied
# dependencies
#sub checkDependencies
#{
#    my $self = shift;

#    foreach my $host (keys %{$self->{REALHOSTS}})
#    {
#        $self->_checkHostDependencies($self->{REALHOSTS}->{$host}->{METRICS});
#    }
#}

=head1 NAME

SiteDB

=head1 DESCRIPTION

The SiteDB module encapsulates the structure containing information
about grid site, services, metricsets & local/remote probes used
for monitoring.

Main component of SiteDB module is list of hosts which belong to site.
Internally SiteDB keeps two lists of hosts in order to handle properly
aliases and LB nodes:
- List HOSTS contains all hosts which are found by NCG::SiteInfo module 
and for these hosts configuration is generated by NCG::ConfigGen. 
- List REALHOSTS contains list of real names of hosts in list HOSTS.

Structure is based on topology and probe description database.
Structure of HOSTS is following:
  $self->{HOSTS}
    - contains list of hosts on site
    - attributes:
      - ADDRESS: host address used for host check
      - ALIAS: alias is set to real hostname if host is an alias
      - REALHOST: real name of host (this is always set to name 
      used in list REALHOSTS)

  $self->{HOSTS}->{$hostName}->{SERVICES}
    - contains list of services on host
    - services are interpreted as grid services in GOCDB (nodetypes)
    - examples: SRM, SE, CE, BDII

  $self->{HOSTS}->{$hostName}->{SERVICES}->{$serviceName}->{VOS}
    - contains list of VOs supported on service
    - this list is not mandatory (e.g. grids without VOs)

  $self->{HOSTS}->{$hostName}->{METRICSETS}
    - contains list of metricsets on host
    - metricsets are interpreted as individual services
    - single service contains several metricsets, overlaps between
    services are allowed (e.g. both CE and SE have gsiftp)
    - examples: MyProxy, globus-GRAM, gsiftp
    - attributes:
      - serviceuri: service uri
      - services: array of services to which metricsets belongs to
        
  $self->{HOSTS}->{$hostName}->{METRICS}
    - contains list of metrics on host
    - metrics are atomic checks for individual metricsets
    - single service contains several metricsets, overlaps are
    not allowed
    - examples: hr.srce.GridFTP-Transfer, ch.cern.LFC-Readdir
    - attributes:
      - TYPE: local|remote
      - DOCURL: URL with metric documentation
      - URL: URL with detailed output (remote only)
      - REMOTESERVICE: name of the remote service which performed
      remote check (remote only)
      - PROBE: executable (local only)
      - NATIVE: name of underlying monitoring system which provides
        native probe (local only)
      - METRICSET: metricset to which metric belongs (local only)
      - CONFIG: configuration paramteres hash (local only)
      - ATTRIBUTES: attributes hash (local only)
      - DEPENDENCIES: array of metrics which metric depends on (local only)

Structure of REALHOSTS is following:
  $self->{REALHOSTS}
    - contains list of hosts on site
    - attributes:
      - ADDRESS: host address used for host check
      
  $self->{REALHOSTS}->{$hostName}->{ATTRIBUTES}
    - contains list of attributes needed for metrics

  $self->{REALHOSTS}->{$hostName}->{ATTRIBUTES}->{$attribute}->{VOS}
    - contains list of attribute values for each supported

  $self->{REALHOSTS}->{$hostName}->{ALIASES}
    - contains list of aliases for the host

  $self->{REALHOSTS}->{$hostName}->{LBNODES}
    - contains list of load balancing nodes for the host

  $self->{REALHOSTS}->{$hostName}->{CONTACTS}
    - contains list of contact persons for host

(Following lists are the same as in list HOSTS with the difference
that memers are aggregated over all aliases)

  $self->{REALHOSTS}->{$hostName}->{SERVICES}
    - contains list of services aggregated over all aliases
    - attributes are the same as in HOSTS list

  $self->{REALHOSTS}->{$hostName}->{SERVICES}->{$serviceName}->{VOS}
    - contains list of VOs supported on service aggregated over all
    aliases
    - attributes are the same as in HOSTS list

  $self->{REALHOSTS}->{$hostName}->{METRICSETS}
    - contains list of metricsets aggregated over all aliases
    - attributes are the same as in HOSTS list
    
  $self->{HOSTS}->{$hostName}->{METRICS}
    - contains list of local-ONLY metrics aggregated over all aliases
    - attributes are the same as in HOSTS list


=head1 SYNOPSIS

  use SiteDB;

=cut

=head1 METHODS

=over

=item C<new>

  $site = SiteDB->new( $attr );

Creates new NCG instance.

=item C<addHost>

  $res = $site->addHost( $host );

Adds new host.

Result is 1 if operation is successful, 0 otherwise.

If $host is not alias, host is added to HOSTS and REALHOSTS. 
If $host is alias, real hostname is added to REALHOSTS. 
If host has LB nodes, LB nodes are added to REALHOSTS.

=item C<addService>

  $res = $ncg->addService( $host, $service );

Adds new service to host. If host doesn't exist, method returns error.

Result is 1 if operation is successful, 0 otherwise.

Service is added to both HOSTS and REALHOSTS.
Services are added to REALHOST in order to generate
proper configuration for LB nodes.

=item C<addVO>

  $res = $ncg->addVO( $host, $service, $vo );

Adds new supported VO to service on host. If service on host doesn't exist,
method returns error.

Result is 1 if operation is successful, 0 otherwise.

VO is added to both HOSTS and REALHOSTS.

=item C<addMetricSet>

  $res = $ncg->addMetricSet( $host, $metricSet, $service );

Adds new metricset to host. If host doesn't exist, method returns error.

Result is 1 if operation is successful, 0 otherwise.

Metricset is added to both HOSTS and REALHOSTS.
Metricsets are added to REALHOST in order to generate
proper configuration for LB nodes.

=item C<addLocalMetric>

  $res = $ncg->addLocalMetric( $host, $metric, $metricSet, $probe,
                               $native, $config, $attributes,
                               $dependencies, $docUrl);

Adds new local metric to host. If host doesn't exist, method returns error.

Parameters are:
    - $host: add metric to defined host
    - $metric: metric name
    - $metricSet: metricset to which metric belongs
    - $probe: executable used for gathering metric
    - $native: name of underlying monitoring system which provides
      native probe
    - $vo: defines if metric is VO dependent
    - $config: configuration paramteres hash (name=>value)
    - $attributes: attributes hash (name=>value)
    - $dependencies: dependencies hash (parentMetric=>value); value is
    1 for host internal dependencies and 0 for external (e.g.
    GridProxy-Valid)
    - $docUrl: URL with metric documentation

Result is 1 if operation is successful, 0 otherwise.

Local metric is added to REALHOSTS only. Local metrics are associated
to metric sets which are associated with all aliases and LB nodes.
Entities in list HOSTS have reference to metrics in REALHOSTS.

=item C<addRemoteMetric>

  $res = $ncg->addRemoteMetric( $host, $metric, $remoteService, $url, $docUrl );

Adds new remote metric to host. If host doesn't exist, method issues warning.

Parameters are:
    - $host: add metric to defined host
    - $metric: metric name
    - $remoteService: name of the remote service which performed
      remote metric
    - $url: URL with detailed output
    - $docUrl: URL with metric documentation

Result is 1 if operation is successful, 0 otherwise.

Remote metric is added to HOSTS only. Remote metrics are associated to reported
names only and not to real host name and LB nodes.

=item C<addLBNode>

  $res = $ncg->addLBNode( $host, $lbnode );

Adds new LB node to host. If host doesn't exist, method issues warning.

Parameters are:
    - $host: add LB node to defined host
    - $lbnode: LB node name

Result is 1 if operation is successful, 0 otherwise.

LB node is added to REALHOSTS only.

=item C<addContact>

  $res = $ncg->addContact( $contact );

Adds contact to all entities on site.

Site level contacts are kept independently from hosts.

=item C<addHostContact>

  $res = $ncg->addHostContact( $host, $contact );

Adds contact to host.

Contact is added to REALHOSTS only because it is reasonable
to expect that the administrator responsible for host is
also responsible for all its aliases.

=item C<removeHost>

  $res = $ncg->removeHost( $host );

Removes host from list HOSTS. Removes host from list of aliases
in list REALHOSTS. Entry in REALHOSTS is kept as long as there
is a single alias left.

=item C<removeMetricSet>

  $res = $ncg->removeMetricSet( $host );

Removes metric set from host in list HOSTS. Removes metricset
from list REALHOSTS and from all aliases in list HOSTS.

=item C<removeLBNode>

  $res = $ncg->removeLBNode( $host, $lbnode );

Removes LB node from host in list REALHOSTS.

=item C<siteName>

  print $ncg->siteName;
  $ncg->siteName('egee.srce.hr');

Accessor method for property SITENAME.

=item C<siteLDAP>

  print $ncg->siteLDAP;
  $ncg->siteLDAP('bdii-egee.srce.hr');

Accessor method for property LDAP_ADDRESS.

=item C<hostAttribute>

  print $ncg->hostAttribute( $host, $attr );
  $ncg->hostAttribute( $host, $attr, $value );

Accessor method for attribute $attr on host $host.

Attributes are kept only in list REALHOSTS.

=item C<hostAttributeVO>

  print $ncg->hostAttributeVO( $host, $attr, $vo );
  $ncg->hostAttributeVO( $host, $attr, $vo, $value );

Accessor method for attribute $attr for VO $vo on host
$host.

Attributes for VOs are kept only in list REALHOSTS.

=item C<hostAttributeVOs>

  @attrs = $ncg->hostAttributeVOs( $host, $attr );

Method returns array of attribute values for each VO.

=item C<metricSetServiceURI>

  print $ncg->metricSetServiceURI( $host, $metricSet );
  $ncg->metricSetServiceURI( $host, $metricSet, $value );

Accessor method for serviceURI for metric set $metricSet
on host $host.

=item C<metric*>

  print $ncg->metric* ( $host, $metric );

Read-only accessors for metric attributes on host $host.

Methods are following: metricType, metricDocUrl, metricProbe
metricNative, metricRemoteService, metricUrl, metricMetricSet,
metricConfig, metricAttributes, metricDependencies.

See DESCRIPTION, C<addLocalMetric> and C<addRemoteMetric> for
detailed description of metric attributes.

=item C<hostAddress>

  print $ncg->hostAddress ( $host );

Read-only accessor for address of host $host.

=item C<hostAlias>

  print $ncg->hostAlias ( $host );

Read-only accessor for alias of host $host.

=item C<LBNodeAddress>

  print $ncg->LBNodeAddress ( $host, $lbnode );

Read-only accessor for address of LB node $lbnod on host $host.

=item C<getHosts>

  @res = $ncg->getHosts( $host );

Retrieves array of hosts in list HOSTS.

This method is useful for generating configuration for
registered hosts.

=item C<getRealHosts>

  @res = $ncg->getRealHosts( $host );

Retrieves array of hosts in list REALHOSTS.

This method is useful for generating configuration for
load balanced nodes.

=item C<getServices>

  @res = $ncg->getServices( $host );

Retrieves array of services associated with $host from
list HOSTS.

=item C<getRealServices>

  @res = $ncg->getRealServices( $host );

Retrieves array of services associated with $host from
list REALHOSTS.

Since entry in REALHOSTS contains aggregated services,
result is different from result of method C<getServices>.

This method is implemented due to a rare case when real
host name is present in list HOSTS and contains different
set of services from the on in list REALHOSTS. Example:
- real.host.org is registered as service CE
- its alias alias.host.org is registered as service MON

=item C<getMetricSets>

  @res = $ncg->getMetricSets( $host );

Retrieves array of metric sets associated with $host from
list HOSTS.

=item C<getRealMetricSets>

  @res = $ncg->getRealMetricSets( $host );

Retrieves array of metric sets associated with $host from
list REALHOSTS.

See comment for C<getRealServices>

=item C<getMetrics>

  @res = $ncg->getMetrics( $host );

Retrieves array of metrics associated with $host from
list HOSTS.

=item C<getLocalMetrics>

  @res = $ncg->getLocalMetrics( $host );

Retrieves array of local metrics associated with $host from
list HOSTS.

=item C<getRemoteMetrics>

  @res = $ncg->getRemoteMetrics( $host );

Retrieves array of remote metrics associated with $host from
list HOSTS.

=item C<getRealMetrics>

  @res = $ncg->getRealMetrics( $host );

Retrieves array of metrics associated with $host from
list REALHOSTS.

See comment for C<getRealServices>

=item C<getLBNodes>

  @res = $ncg->getLBNodes( $host );

Retrieves array of LB nodes associated with host $host.

Array is retrieved from entry in REALHOSTS, but the 
argument $host can be both from HOSTS and REALHOSTS.
In case when alias is used, method will find the
appropriate entry in REALHOSTS by using REAL_HOST
attribute.

=item C<getAliases>

  @res = $ncg->getAliases( $host );

Retrieves array of aliases associated with host $host.

See comment for C<getLBNodes>

=item C<getContacts>

  @res = $ncg->getContacts;

Retrieves array of global contacts associated with 
all hosts.

=item C<getHostContacts>

  @res = $ncg->getHostContacts ( $host );

Retrieves array of contacts associated host $host

See comment for C<getLBNodes>

=item C<hasHost>

  $res = $ncg->hasHost ( $host );

Method checks if $host exists in list HOSTS.

=item C<hasRealHost>

  $res = $ncg->hasRealHost ( $host );

Method checks if $host exists in list REALHOSTS.

=item C<hasService>

  $res = $ncg->hasService ( $host, $service );

Method checks if $host in list HOSTS has $service.

=item C<hasRealService>

  $res = $ncg->hasRealService ( $host, $service );

Method checks if $host in list REALHOSTS has $service.

=item C<hasMetricSet>

  $res = $ncg->hasMetricSet ( $host, $metricSet );

Method checks if $host in list HOSTS has $metricSet.

=item C<hasRealMetricSet>

  $res = $ncg->hasRealMetricSet ( $host, $metricSet );

Method checks if $host in list REALHOSTS has $metricSet.

=item C<hasAttribute>

  $res = $ncg->hasAttribute ( $host, $attribute );

Method checks if $host has $attribute defined.

=item C<hasServiceURI>

  $res = $ncg->hasAttribute ( $host, $metricSet );

Method checks if $host has service uri defined
for metric set $metricSet.

=item C<hasContacts>

  $res = $ncg->hasContacts ( $host );

Method checks if $host has contacts defined.

=item C<hasVO>

  $res = $ncg->hasVO ( $host, $vo );

Method checks if there is any service on $host which supports 
$vo. 

In case when all services don't have VOs defined, method
returns true. Example is BDII service.

=item C<hasLBNodes>

  $res = $ncg->hasLBNodes ( $host );

Method checks if $host has LB nodes.

=back

=head1 SEE ALSO

  NCG

=cut


1;
