#!/usr/bin/perl -w
#
# Nagios configuration generator (WLCG probe based)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

use Config::General;
use Getopt::Long;
use SiteDB;
use strict;

my $DEFAULT_CONFIG_FILE = "/opt/lcg/etc/ncg/ncg.conf";
my $VERBOSE;

sub verbose {
    my $msg = shift;
    print $msg if $VERBOSE;
}

sub getGliteVersion {
    my $res=`glite-version 2>&1`;
    unless ($?) {
        chop $res;
    } else {
        $res = "UNKNOWN";
    }

    return $res;
}

sub _invokeNCGObject {
    my $conf = shift || return;
    my $parent = shift || return;
    my $child = shift || return;
    my $siteDB = shift || return;

    my $confRef = $conf->{$parent}->{$child};
    my @confArr;
    my $objName = $parent . "::" . $child;
    my $obj;

    eval "use $objName;";
    if ($@) {
        verbose "ERROR: cannot use module $objName, error: $@\n";
        return;
    }

    if(ref($confRef) eq "ARRAY") {
        @confArr = @{$confRef};
    } else {
        @confArr = ($confRef);
    }

    foreach my $instance (@confArr) {
         verbose("Invoking $objName.\n");
         $instance->{SITEDB} = $siteDB;
         
         # hack to set GLITE_VERSION if missing
         if ( exists $instance->{GLITE_VERSION} && !$instance->{GLITE_VERSION}) {
            $instance->{GLITE_VERSION} = $conf->{GLITE_VERSION};
         }

         $obj = new $objName ($instance);
         next if (!$obj);
         $obj->getData();
    }
}

sub invokeNCGObjects {
    my $conf = shift || return;
    my $parent = shift || return;
    my $siteDB = shift || return;

    return if (! exists $conf->{$parent});

    foreach my $confKey ( keys %{$conf->{$parent}} ) {
        _invokeNCGObject($conf, $parent, $confKey, $siteDB);
    }
}

sub invokeNCGObject {
    my $conf = shift || return;
    my $parent = shift || return;
    my $child = shift || return;
    my $siteDB = shift || return;
    return if (! exists $conf->{$parent});

    if (exists $conf->{$parent}->{$child}) {
        _invokeNCGObject($conf, $parent, $child, $siteDB);
        delete $conf->{$parent}->{$child};
    }
}

#################################
#   Main program
#################################

my $help;
my $CONFIG_FILE;
my $DEBUG;

if (!GetOptions (
    'h' => \$help, 'help' => \$help,
    'v' => \$VERBOSE, 'verbose' => \$VERBOSE,
    'd' => \$DEBUG, 'debug' => \$DEBUG,
    'config=s' => \$CONFIG_FILE ) ) {
    exit -1;
}

if ($help) {
    print "
Nagios configuration generator (WLCG probe based)

    Options:
        --config STRING
            Path to generated configuration file.
            For detailed description of options for
            individual modules see perldoc.
            (default: $DEFAULT_CONFIG_FILE)

        -h, --help
            Basic program description

        -v, --verbose
            Print verbose logging information

        -d, --debug
            Print detailed program execution flow

";
    exit 0;
}

if (!defined $CONFIG_FILE) {
    $CONFIG_FILE = $DEFAULT_CONFIG_FILE;
}

my %options = (-ConfigFile => $CONFIG_FILE, -AllowMultiOptions => 1, -InterPolateVars => 1, -InterPolateEnv => 1);
my $config = new Config::General(%options);
if (!$config) {
    print "Error reading configuration file!\n";
    exit -1;
}
my %conf = $config->getall;

my $siteDB = new SiteDB({SITENAME=>$conf{SITENAME},VERBOSE=>$VERBOSE,DEBUG=>$DEBUG});
if (!$siteDB) {
    print "Error creating SiteDB structure!\n";
    exit -1;
}

if (!$conf{GLITE_VERSION}) {
    $conf{GLITE_VERSION} = getGliteVersion();
}

# invoke SAM first in order to get BDII if not defined
invokeNCGObject (\%conf, "NCG::SiteInfo", "SAM", $siteDB );

# invoke others...
invokeNCGObjects (\%conf, "NCG::SiteInfo", $siteDB);
invokeNCGObjects (\%conf, "NCG::LocalMetricSets", $siteDB);

# invoke LDAP first otherwise Active will erase SRMs because of
# missing SE_PATH
invokeNCGObject (\%conf, "NCG::LocalMetricsAttrs", "LDAP", $siteDB );

# invoke others...
invokeNCGObjects (\%conf, "NCG::LocalMetricsAttrs", $siteDB);
invokeNCGObjects (\%conf, "NCG::LocalRules", $siteDB);
invokeNCGObjects (\%conf, "NCG::LocalMetrics", $siteDB);
invokeNCGObjects (\%conf, "NCG::RemoteMetrics", $siteDB);
invokeNCGObjects (\%conf, "NCG::ConfigGen", $siteDB);




